ApplicationModel subclass: #TaxForm
	instanceVariableNames: 'business businessLoss childCare city disability employment farming firstName infoChanged lastName limit1 limit2 limit3 marriedDeduction marriedStatus moving pension pensionPlan postalCode printed rate1 rate2 rate3 rental saved street taxableIncome totalDeductions totalIncome totalTax unionDues married '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'App 1'!
TaxForm comment:
'This class is an example used in Ivan Tomek: The Joy of Smalltalk. It demonstrates porinciples of user interfaces with simple widgets on a simplified tax return form.

All variables are ValueHolders on fixed point numbers, Booleans, or Strings. The class and purpose are mostly obvious and left further undefined.

business 		income
businessLoss 	deduction
childCare 		income
city 			<String>
disability 		income
employment 	income
farming 			income
firstName 		<String>
infoChanged		<Boolean>
lastName 		<Boolean>
limit1			limit of first tax bracket
limit2			limit of second tax bracket
limit3 			limit of third tax bracket
marriedDeduction
marriedStatus	<Symbol> #married, #separated, #single, #widowed
moving			deduction
pension 		income
pensionPlan 	deduction
postalCode		<String>
rate1			tax rate for bracket1
rate2			tax rate for bracket2
rate3			tax rate for bracket3
rental 			income
street 			<String>
taxableIncome 	calculated
totalDeductions 	calculated
totalIncome 		calculated
totalTax 		calculated
unionDues 		deduction'!


!TaxForm methodsFor: 'initialization'!

initialize
	self initializeVars.
	self initializeChanges!

initializeChanges
	business onChangeSend: #newTotals to: self.
	businessLoss onChangeSend: #newTotals to: self.
	childCare onChangeSend: #newTotals to: self.
	disability onChangeSend: #newTotals to: self.
	employment onChangeSend: #newTotals to: self.
	farming onChangeSend: #newTotals to: self.
	limit1 onChangeSend: #newTotals to: self.
	limit2 onChangeSend: #newTotals to: self.
	limit3 onChangeSend: #newTotals to: self.
	marriedStatus onChangeSend: #newTotals to: self.
	moving onChangeSend: #newTotals to: self.
	pension onChangeSend: #newTotals to: self.
	pensionPlan onChangeSend: #newTotals to: self.
	rate1 onChangeSend: #newTotals to: self.
	rate2 onChangeSend: #newTotals to: self.
	rate3 onChangeSend: #newTotals to: self.
	rental onChangeSend: #newTotals to: self.
	unionDues onChangeSend: #newTotals to: self!

initializeVars
	business := ValueHolder newFraction.
	businessLoss := ValueHolder newFraction.
	childCare := ValueHolder newFraction.
	city := ValueHolder newString.
	disability := ValueHolder newFraction.
	employment := ValueHolder newFraction.
	farming := ValueHolder newFraction.
	firstName := ValueHolder newString.
	lastName := ValueHolder newString.
	limit1 := 5000 asValue.
	limit2 := 20000 asValue.
	limit3 := 40000 asValue.
	marriedDeduction := ValueHolder newFraction.
	marriedStatus := #single asValue.
	moving := ValueHolder newFraction.
	pension := ValueHolder newFraction.
	pensionPlan := ValueHolder newFraction.
	postalCode := ValueHolder newString.
	rate1 := 0.1 asValue.
	rate2 := 0.2 asValue.
	rate3 := 0.3 asValue.
	rental := ValueHolder newFraction.
	street := ValueHolder newString.
	taxableIncome := ValueHolder newFraction.
	totalDeductions := ValueHolder newFraction.
	totalIncome := ValueHolder newFraction.
	totalTax := ValueHolder newFraction.
	unionDues := ValueHolder newFraction.
	infoChanged := true.
	printed := false.
	saved := false! !

!TaxForm methodsFor: 'aspects'!

business
	^business!

businessLoss
	^businessLoss!

childCare
	^childCare!

city
	^city!

disability
	^disability!

employment
	^employment!

farming
	^farming!

firstName
	^firstName!

lastName
	^lastName!

married
	^marriedDeduction!

marriedStatus
	^marriedStatus!

moving
	^moving!

pension
	^pension!

pensionPlan
	^pensionPlan!

postalCode
	^postalCode!

rental
	^rental!

street
	^street!

taxableIncome
	^taxableIncome!

totalDeductions
	^totalDeductions!

totalIncome
	^totalIncome!

totalTax
	^totalTax!

unionDues
	^unionDues! !

!TaxForm methodsFor: 'closing'!

changeRequest
	| isIncomplete |
	isIncomplete := city value isEmpty or: [firstName value isEmpty or: [lastName value isEmpty or: [postalCode value isEmpty or: [street value isEmpty]]]].
	isIncomplete ifTrue: [^Dialog confirm: 'The form is not complete. Close anyway?'].
	(printed or: [saved])
		ifFalse: [^Dialog confirm: 'The form has not been printed or saved. Close anyway?'].
	^true! !

!TaxForm methodsFor: 'actions'!

brackets
	limit1 value: (Dialog request: 'Enter limit for lowest bracket' initialAnswer: '5000') asNumber.
	limit2 value: (Dialog request: 'Enter limit for middle bracket' initialAnswer: '20000') asNumber.
	limit3 value: (Dialog request: 'Enter limit for highest bracket' initialAnswer: '40000') asNumber!

help
	| string |
	string := 'At the top, enter personal information.
In the left column, enter all categories of income that apply.
In the right column, enter all applicable expenses.
The buttons on the right allow you to
- print a tax former on a printer,
- change limits of tax categories and their corresponding tax rates,
- save your form in a file,
- load a form saved in a file, and
- close (quit) this program.

Note that you can use the Tab key to move to the next item.'.
	(SimpleHelp helpString: string) open!

load
	"load directory and info"

	| bos fileName |
	fileName := Filename named: (Dialog
					requestFileName: 'Load from file:'
					default: 'mytax.txf'
					version: #mustBeOld
					ifFail: [^self]).
	fileName exists ifFalse: [^Dialog warn: 'Data file not found'].
	bos := BinaryObjectStorage onOld: fileName readStream.
	business value: bos next.
	businessLoss value: bos next.
	childCare value: bos next.
	city value: bos next.
	disability value: bos next.
	employment value: bos next.
	farming value: bos next.
	firstName value: bos next.
	lastName value: bos next.
	limit1 value: bos next.
	limit2 value: bos next.
	limit3 value: bos next.
	marriedStatus value: bos next.
	moving value: bos next.
	pension value: bos next.
	pensionPlan value: bos next.
	postalCode value: bos next.
	rate1 value: bos next.
	rate2 value: bos next.
	rate3 value: bos next.
	rental value: bos next.
	street value: bos next.
	unionDues value: bos next.
	bos close.
	infoChanged := false.
	self newTotals!

print
	| string |
	string := '
Simple tax report for ' , firstName value , ' ' , lastName value , (marriedStatus == #married
					ifTrue: ['married']
					ifFalse: ['unmarried']) , '
Address: ' , street value , ' ' , city , ' ' , postalCode value , '

Income:
employment		' , employment value , 'business		' , business value , 'farming			' , farming value , 'disability		' , disability value , 'pension			' , pension value , 'rental			' , rental value , 'total income	' , totalIncome , '
Deductions:
business loss	' , businessLoss value , 'child care		' childCare value , 'moving			' , moving value , 'pension plan	' , pensionPlan value , 'taxable income	' , taxableIncome value , 'union dues		' , unionDues value , 'married			' , married value , 'total deduction	' , totalDeductions value , '

Total taxable income	' , taxableIncome value , ' TOTAL TAX			' totalTax value.
	string asComposedText hardcopy.
	printed := true!

quit
	self closeRequest!

rates
	rate1 value: (Dialog request: 'Enter rate for lowest bracket' initialAnswer: '0.1') asNumber.
	rate2 value: (Dialog request: 'Enter rate for middle bracket' initialAnswer: '0.2') asNumber.
	rate3 value: (Dialog request: 'Enter rate for highest bracket' initialAnswer: '0.3') asNumber!

save
	"Save information in files"

	| fileName |
	fileName := Dialog
				requestFileName: 'Save to file:'
				default: 'mytax.txf'
				version: #any
				ifFail: [^self].
	(BinaryObjectStorage onNew: (Filename named: fileName) writeStream)
		nextPut: business value;
		nextPut: businessLoss value;
		nextPut: childCare value;
		nextPut: city value;
		nextPut: disability value;
		nextPut: employment value;
		nextPut: farming value;
		nextPut: firstName value;
		nextPut: lastName value;
		nextPut: limit1 value;
		nextPut: limit2 value;
		nextPut: limit3 value;
		nextPut: marriedStatus value;
		nextPut: moving value;
		nextPut: pension value;
		nextPut: pensionPlan value;
		nextPut: postalCode value;
		nextPut: rate1 value;
		nextPut: rate2 value;
		nextPut: rate3 value;
		nextPut: rental value;
		nextPut: street value;
		nextPut: unionDues value; close.
	infoChanged := false.
	saved := true! !

!TaxForm methodsFor: 'changes'!

newTotals
	| bracket1 bracket2 bracket3 taxable |
	totalIncome value: business value + disability value + employment value + farming value + pension value + rental value.
	marriedDeduction value: (marriedStatus value == #married
			ifTrue: [500]
			ifFalse: [0]).
	totalDeductions value: pensionPlan value + unionDues value + businessLoss value + childCare value + moving value + marriedDeduction value.
	taxableIncome value: ((totalIncome value - totalDeductions value) max: 0).
	taxable := taxableIncome value.
	bracket1 := taxable > limit1 value
				ifTrue: [taxable - limit1 value min: limit2 value - limit1 value]
				ifFalse: [0].
	bracket2 := taxable > limit2 value
				ifTrue: [taxable - limit2 value min: limit3 value - limit2 value]
				ifFalse: [0].
	bracket3 := taxable > limit3 value
				ifTrue: [taxable - limit3 value]
				ifFalse: [0].
	totalTax value: bracket1 * rate1 value + (bracket2 * rate2 value) + (bracket3 * rate3 value).
	infoChanged := true.
	printed := false.
	saved := false! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TaxForm class
	instanceVariableNames: ''!


!TaxForm class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Simplified Tax Return' 
			#min: #(#Point 40 20 ) 
			#bounds: #(#Rectangle 265 31 756 560 ) 
			#colors: 
			#(#LookPreferences 
				#setBackgroundColor: #(#ColorValue 7167 8191 8191 ) ) ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#RegionSpec 
					#layout: #(#LayoutFrame 0 0.672098 0 0.25 0 0.98167 0 0.307143 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#lineWidth: 1 ) 
				#(#RegionSpec 
					#layout: #(#LayoutFrame 0 0.00814664 0 0.707143 0 0.659878 0 0.9 ) 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #pink ) 
						#setBackgroundColor: #(#ColorValue #pink ) ) ) 
				#(#RegionSpec 
					#layout: #(#LayoutFrame 0 0.00814664 0 0.892857 0 0.659878 0 0.992857 ) 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #red ) 
						#setBackgroundColor: #(#ColorValue 8191 7167 6143 ) ) 
					#lineWidth: 4 ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.01222 0 0.020794 ) 
					#label: 'First Name' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.01222 0 0.0535714 0 0.321792 0 0.0964286 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #firstName ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.342159 0 0.0535714 0 0.651731 0 0.0964286 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #lastName ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.342159 0 0.020794 ) 
					#label: 'Last Name' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.01222 0 0.100189 ) 
					#label: 'Street' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.01222 0 0.132143 0 0.321792 0 0.175 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #street ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.342159 0 0.100189 ) 
					#label: 'City' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.672098 0 0.132143 0 0.98167 0 0.175 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #postalCode ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.672098 0 0.1 ) 
					#label: 'Postal code' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.668024 0 0.0321429 ) 
					#model: #marriedStatus 
					#label: 'Married' 
					#select: #married ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.668024 0 0.0714286 ) 
					#model: #marriedStatus 
					#label: 'Single' 
					#select: #single ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.822811 0 0.0321429 ) 
					#model: #marriedStatus 
					#label: 'Widowed' 
					#select: #widowed ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.822811 0 0.0714286 ) 
					#model: #marriedStatus 
					#label: 'Separated' 
					#select: #separated ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.114053 0 0.189286 ) 
					#label: 'INCOME' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.01222 0 0.221172 ) 
					#label: 'Employment' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.01222 0 0.253571 0 0.321792 0 0.296429 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #employment 
					#alignment: #center 
					#type: #fixedpoint 
					#formatString: '$#,##0.00;[Red]-$#,##0.00' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.01222 0 0.3 ) 
					#label: 'Business' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.01222 0 0.332143 0 0.321792 0 0.375 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #business 
					#alignment: #center 
					#type: #fixedpoint 
					#formatString: '$#,##0.00;[Red]-$#,##0.00' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.01222 0 0.382143 ) 
					#label: 'Old Age Pension' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.01222 0 0.414286 0 0.321792 0 0.457143 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #pension 
					#alignment: #center 
					#type: #fixedpoint 
					#formatString: '$#,##0.00;[Red]-$#,##0.00' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.01222 0 0.460714 ) 
					#label: 'Disability Benefits' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.01222 0 0.492857 0 0.321792 0 0.535714 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #disability 
					#alignment: #center 
					#type: #fixedpoint 
					#formatString: '$#,##0.00;[Red]-$#,##0.00' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.01222 0 0.542857 ) 
					#label: 'Rental Income' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.01222 0 0.575 0 0.321792 0 0.617857 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #rental 
					#alignment: #center 
					#type: #fixedpoint 
					#formatString: '$#,##0.00;[Red]-$#,##0.00' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.01222 0 0.621429 ) 
					#label: 'Farming Income' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.01222 0 0.653571 0 0.321792 0 0.696429 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #farming 
					#alignment: #center 
					#type: #fixedpoint 
					#formatString: '$#,##0.00;[Red]-$#,##0.00' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.01222 0 0.707143 ) 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #black ) ) 
					#label: 'Total Income' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.01222 0 0.746429 0 0.321792 0 0.789286 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #totalIncome 
					#alignment: #center 
					#isReadOnly: true 
					#type: #fixedpoint 
					#formatString: '$#,##0.00;-$#,##0.00' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.403259 0 0.189286 ) 
					#label: 'DEDUCTIONS' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.342159 0 0.221172 ) 
					#label: 'Pension Plan' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.342159 0 0.253571 0 0.651731 0 0.296429 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #pensionPlan 
					#alignment: #center 
					#type: #fixedpoint 
					#formatString: '$#,##0.00;[Red]-$#,##0.00' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.342159 0 0.3 ) 
					#label: 'Married Deduction' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.342159 0 0.332143 0 0.651731 0 0.375 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #pink ) ) 
					#model: #married 
					#tabable: false 
					#alignment: #center 
					#type: #fixedpoint 
					#formatString: '$#,##0.00;-$#,##0.00' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.342159 0 0.382143 ) 
					#label: 'Union/professional dues' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.342159 0 0.414286 0 0.651731 0 0.457143 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #unionDues 
					#alignment: #center 
					#type: #fixedpoint 
					#formatString: '$#,##0.00;[Red]-$#,##0.00' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.342159 0 0.460714 ) 
					#label: 'Moving Expenses' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.342159 0 0.492857 0 0.651731 0 0.535714 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #moving 
					#alignment: #center 
					#type: #fixedpoint 
					#formatString: '$#,##0.00;[Red]-$#,##0.00' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.342159 0 0.542857 ) 
					#label: 'Child Care' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.342159 0 0.575 0 0.651731 0 0.617857 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #childCare 
					#alignment: #center 
					#type: #fixedpoint 
					#formatString: '$#,##0.00;[Red]-$#,##0.00' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.342159 0 0.621429 ) 
					#label: 'Business loss' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.342159 0 0.653571 0 0.651731 0 0.696429 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #businessLoss 
					#alignment: #center 
					#type: #fixedpoint 
					#formatString: '$#,##0.00;[Red]-$#,##0.00' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.342159 0 0.707143 ) 
					#label: 'Total Deductions' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.342159 0 0.746429 0 0.651731 0 0.789286 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #totalDeductions 
					#alignment: #center 
					#isReadOnly: true 
					#type: #fixedpoint 
					#formatString: '$#,##0.00;-$#,##0.00' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.0244399 0 0.807143 ) 
					#label: 'Taxable Income = Total Income - Total Deductions' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.175153 0 0.839286 0 0.484725 0 0.882143 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #taxableIncome 
					#alignment: #center 
					#isReadOnly: true 
					#type: #fixedpoint 
					#formatString: '$#,##0.00;-$#,##0.00' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.211813 0 0.892857 ) 
					#label: 'Total Tax from Tables' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.175153 0 0.932143 0 0.484725 0 0.975 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #totalTax 
					#alignment: #center 
					#isReadOnly: true 
					#type: #fixedpoint 
					#formatString: '$#,##0.00;-$#,##0.00' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.692464 0 0.55 0 0.96945 0 0.592857 ) 
					#model: #print 
					#tabable: false 
					#label: 'Print' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.692464 0 0.742857 0 0.96945 0 0.785714 ) 
					#model: #load 
					#tabable: false 
					#label: 'Load from File' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.692464 0 0.807143 0 0.96945 0 0.85 ) 
					#model: #save 
					#tabable: false 
					#label: 'Save to File' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.692464 0 0.871428 0 0.96945 0 0.914286 ) 
					#model: #help 
					#tabable: false 
					#label: 'Help' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.692464 0 0.935714 0 0.96945 0 0.978571 ) 
					#model: #quit 
					#tabable: false 
					#label: 'Quit' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.682281 0 0.257143 ) 
					#label: 'White space - your input' ) 
				#(#RegionSpec 
					#layout: #(#LayoutFrame 0 0.672098 0 0.314286 0 0.98167 0 0.371429 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #pink ) ) 
					#lineWidth: 1 ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.690428 0 0.321429 ) 
					#label: 'Pink space - automatic' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.692464 0 0.614286 0 0.96945 0 0.657143 ) 
					#model: #rates 
					#tabable: false 
					#label: 'Change tax rates' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.692464 0 0.678571 0 0.96945 0 0.721429 ) 
					#model: #brackets 
					#tabable: false 
					#label: 'Change tax brackets' 
					#defaultable: true ) 
				#(#GroupBoxSpec 
					#layout: #(#LayoutFrame 0 0.676171 0 0.514286 0 0.985743 0 0.992857 ) 
					#label: 'Commands' ) 
				#(#GroupBoxSpec 
					#layout: #(#LayoutFrame 0 0.659878 0 0.210714 0 0.991853 0 0.382143 ) 
					#label: 'Instructions' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.342159 0 0.132143 0 0.651731 0 0.175 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #city ) ) ) )! !

!TaxForm class methodsFor: 'opening'!

opening
	"TaxForm open"! !

ApplicationModel subclass: #RadioButtonTest
	instanceVariableNames: 'case text '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'App 1'!


!RadioButtonTest methodsFor: 'actions'!

enterText
	text := Dialog request: 'Please enter text' initialAnswer: ''! !

!RadioButtonTest methodsFor: 'aspects'!

case
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^case isNil
		ifTrue:
			[case := nil asValue]
		ifFalse:
			[case]! !

!RadioButtonTest methodsFor: 'displaying'!

display
	"Display text using selected style but leave initial text unchanged."

	"Calculate text using selected style."
	| newText |
	newText := case value == #unchanged
				ifTrue: [text]
				ifFalse: [case value == #capitalize
						ifTrue: [text asUppercase]
						ifFalse: [text asLowercase]].	
	"Display in Transcript."
	Transcript cr; show: newText! !

!RadioButtonTest methodsFor: 'initialization'!

initialize
	case := #capitalize asValue! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

RadioButtonTest class
	instanceVariableNames: ''!


!RadioButtonTest class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Unlabeled Canvas' 
			#bounds: #(#Rectangle 419 325 691 424 ) ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#RadioButtonSpec 
					#layout: #(#Point 8 10 ) 
					#model: #case 
					#label: 'unchanged' 
					#select: #unchanged ) 
				#(#RadioButtonSpec 
					#layout: #(#Point 8 40 ) 
					#model: #case 
					#label: 'capitalize' 
					#select: #capitalize ) 
				#(#RadioButtonSpec 
					#layout: #(#Point 8 70 ) 
					#model: #case 
					#label: 'lower case' 
					#select: #lowerCase ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 149 18 252 43 ) 
					#model: #enterText 
					#label: 'Enter text' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 149 57 252 82 ) 
					#model: #display 
					#label: 'Display' 
					#defaultable: true ) ) ) )! !

ApplicationModel subclass: #InputFieldTest
	instanceVariableNames: 'inputField outputField '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'App 1'!


!InputFieldTest methodsFor: 'aspects'!

inputField
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^inputField isNil
		ifTrue:
			[inputField := String new asValue]
		ifFalse:
			[inputField]!

outputField
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^outputField isNil
		ifTrue:
			[outputField := String new asValue]
		ifFalse:
			[outputField]! !

!InputFieldTest methodsFor: 'initialization'!

initialize
	inputField := '' asValue.	"The value will be displayed when the window opens."
	inputField onChangeSend: #newInput to: self! !

!InputFieldTest methodsFor: 'changes'!

newInput
	outputField value: inputField value! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

InputFieldTest class
	instanceVariableNames: ''!


!InputFieldTest class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Input Fields' 
			#bounds: #(#Rectangle 404 277 604 443 ) 
			#isEventDriven: true ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#LabelSpec 
					#layout: #(#Point 66 12 ) 
					#label: 'Input field' ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 47 44 145 63 ) 
					#model: #inputField ) 
				#(#LabelSpec 
					#layout: #(#Point 61 93 ) 
					#label: 'Output field' ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 46 125 147 144 ) 
					#model: #outputField 
					#isReadOnly: true ) ) ) )! !

ApplicationModel subclass: #CheckBoxTest
	instanceVariableNames: 'sin square cos cube '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'App 1'!


!CheckBoxTest methodsFor: 'aspects'!

cos
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^cos isNil
		ifTrue:
			[cos := false asValue]
		ifFalse:
			[cos]!

cube
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^cube isNil
		ifTrue:
			[cube := false asValue]
		ifFalse:
			[cube]!

sin
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^sin isNil
		ifTrue:
			[sin := false asValue]
		ifFalse:
			[sin]!

square
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^square isNil
		ifTrue:
			[square := false asValue]
		ifFalse:
			[square]! !

!CheckBoxTest methodsFor: 'actions'!

display
	"Check which boxes are on, calculate corresponding values, and display them in Transcript. 
	Don't print anything if no box is selected."

	(sin value or: [cos value or: [square value or: [cube value]]])
		ifFalse: [^self].	
"For all values in the range, calculate selected functions and print in Transcript."
	Transcript clear.
	sin value ifTrue: [Transcript show: 'sin'; tab].
	cos value ifTrue: [Transcript show: 'cos'; tab].
	cube value ifTrue: [Transcript show: 'cube'; tab].
	square value ifTrue: [Transcript show: 'square'; cr].
	1 to: 20 do: [:argument | "Examine all check boxes to determine which functions to print."
			Transcript show: argument printString; tab.
			sin value ifTrue: [Transcript show: argument sin printString; tab].
			cos value ifTrue: [Transcript show: argument cos printString; tab].
			cube value ifTrue: [Transcript show: (argument squared * argument) printString; tab].
			square value ifTrue: [Transcript show: argument squared printString; cr]]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

CheckBoxTest class
	instanceVariableNames: ''!


!CheckBoxTest class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Function Tables' 
			#min: #(#Point 200 130 ) 
			#max: #(#Point 200 130 ) 
			#bounds: #(#Rectangle 300 200 500 330 ) 
			#isEventDriven: true ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#CheckBoxSpec 
					#layout: #(#Point 29 18 ) 
					#model: #sin 
					#label: 'sin' ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 122 18 ) 
					#model: #cos 
					#label: 'cos' ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 29 65 ) 
					#model: #square 
					#label: 'square' ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 122 65 ) 
					#model: #cube 
					#label: 'cube' ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 67 95 119 120 ) 
					#model: #display 
					#label: 'Display' 
					#defaultable: true ) ) ) )! !

ApplicationModel subclass: #DriveInMenu
	instanceVariableNames: 'pineApple crepes basic total lentil brownies figs coke gazpacho chicken halibut mozarella applePie appleJuice mozzarella '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'App 1'!


!DriveInMenu methodsFor: 'actions'!

ok
	Dialog warn: 'Your order is being processed'! !

!DriveInMenu methodsFor: 'aspects'!

appleJuice
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^appleJuice isNil
		ifTrue:
			[appleJuice := false asValue]
		ifFalse:
			[appleJuice]!

applePie
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^applePie isNil
		ifTrue:
			[applePie := false asValue]
		ifFalse:
			[applePie]!

basic
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^basic isNil
		ifTrue:
			[basic := false asValue]
		ifFalse:
			[basic]!

brownies
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^brownies isNil
		ifTrue:
			[brownies := false asValue]
		ifFalse:
			[brownies]!

chicken
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^chicken isNil
		ifTrue:
			[chicken := false asValue]
		ifFalse:
			[chicken]!

coke
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^coke isNil
		ifTrue:
			[coke := false asValue]
		ifFalse:
			[coke]!

crepes
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^crepes isNil
		ifTrue:
			[crepes := false asValue]
		ifFalse:
			[crepes]!

figs
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^figs isNil
		ifTrue:
			[figs := false asValue]
		ifFalse:
			[figs]!

gazpacho
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^gazpacho isNil
		ifTrue:
			[gazpacho := false asValue]
		ifFalse:
			[gazpacho]!

halibut
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^halibut isNil
		ifTrue:
			[halibut := false asValue]
		ifFalse:
			[halibut]!

lentil
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^lentil isNil
		ifTrue:
			[lentil := false asValue]
		ifFalse:
			[lentil]!

mozzarella
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^mozzarella isNil
		ifTrue:
			[mozzarella := false asValue]
		ifFalse:
			[mozzarella]!

pineApple
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^pineApple isNil
		ifTrue:
			[pineApple := false asValue]
		ifFalse:
			[pineApple]!

total
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^total isNil
		ifTrue:
			[total := String new asValue]
		ifFalse:
			[total]! !

!DriveInMenu methodsFor: 'initialization'!

initialize
	self initializeVariables.
	self initializeChanges!

initializeChanges
	applePie onChangeSend: #checkApplePie to: self.
	appleJuice onChangeSend: #checkAppleJuice to: self.
	basic onChangeSend: #checkBasic to: self.
	brownies onChangeSend: #checkBrownies to: self.
	chicken onChangeSend: #checkChicken to: self.
	coke onChangeSend: #checkCoke to: self.
	crepes onChangeSend: #checkCrepes to: self.
	figs onChangeSend: #checkFigs to: self.
	gazpacho onChangeSend: #checkGazpacho to: self.
	halibut onChangeSend: #checkHalibut to: self.
	lentil onChangeSend: #checkLentil to: self.
	mozzarella onChangeSend: #checkMozzarella to: self.
	pineApple onChangeSend: #checkPineapple to: self!

initializeVariables
	applePie := false asValue.
	appleJuice := false asValue.
	basic := false asValue.
	brownies := false asValue.
	chicken := false asValue.
	coke := false asValue.
	crepes := false asValue.
	figs := false asValue.
	gazpacho := false asValue.
	halibut := false asValue.
	lentil := false asValue.
	mozzarella := false asValue.
	pineApple := false asValue.
	total := 0.0 asValue! !

!DriveInMenu methodsFor: 'changes'!

checkAppleJuice
	appleJuice value
		ifTrue: [total value: (total value  + 0.70) ]
		ifFalse: [total value: (total value  - 0.70) ]!

checkApplePie
	applePie value
		ifTrue: [total value: (total value  + 3.7) ]
		ifFalse: [total value: (total value  - 3.7) ]!

checkBasic
	basic value
		ifTrue: 
			[total value: total value + 3.5.
			(self builder componentAt: #mozzarella) enable.
			(self builder componentAt: #figs) enable.
			(self builder componentAt: #pineApple) enable.
			(self builder componentAt: #mozzarellaPrice) enable.
			(self builder componentAt: #figsPrice) enable.
			(self builder componentAt: #pineapplePrice) enable]
		ifFalse: 
			[total value: total value - 3.5.
			(self builder componentAt: #mozzarella) disable.
			(self builder componentAt: #figs) disable.
			(self builder componentAt: #pineApple) disable.
			(self builder componentAt: #mozzarellaPrice) disable.
			(self builder componentAt: #figsPrice) disable.
			(self builder componentAt: #pineapplePrice) disable.
			mozarella value ifTrue: [mozarella value: false].
			figs value ifTrue: [figs value: false].
			pineApple value ifTrue: [pineApple value: false]]!

checkBrownies
	brownies value
		ifTrue: [total value: (total value  + 2.50) ]
		ifFalse: [total value: (total value  - 2.50) ]!

checkChicken
	chicken value
		ifTrue: [total value: (total value  + 7.25) ]
		ifFalse: [total value: (total value  - 7.25) ]!

checkCoke
	coke value
		ifTrue: [total value: (total value  + 0.80) ]
		ifFalse: [total value: (total value  - 0.80) ]!

checkCrepes
	crepes value
		ifTrue: [total value: (total value  + 4.10) ]
		ifFalse: [total value: (total value  - 4.10) ]!

checkFigs
	figs value
		ifTrue: [total value: (total value  + 1.25) ]
		ifFalse: [total value: (total value  - 1.25) ]!

checkGazpacho
	gazpacho value
		ifTrue: [total value: (total value  + 3.5) ]
		ifFalse: [total value: (total value  - 3.5) ]!

checkHalibut
	halibut value
		ifTrue: [total value: (total value  + 6.95) ]
		ifFalse: [total value: (total value  - 6.95) ]!

checkLentil
	lentil value
		ifTrue: [total value: (total value  + 2.5) ]
		ifFalse: [total value: (total value  - 2.5) ]!

checkMozzarella
	mozzarella value
		ifTrue: [total value: (total value  + 1.15) ]
		ifFalse: [total value: (total value  - 1.15) ]!

checkPineapple
	pineApple value
		ifTrue: [total value: (total value  + 1.0) ]
		ifFalse: [total value: (total value  - 1.0) ]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

DriveInMenu class
	instanceVariableNames: ''!


!DriveInMenu class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Jana''s drive-in menu' 
			#bounds: #(#Rectangle 222 70 578 531 ) ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#LabelSpec 
					#layout: #(#Point 8 8 ) 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #red ) ) 
					#label: 'Soup' ) 
				#(#LabelSpec 
					#layout: #(#Point 8 64 ) 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #red ) ) 
					#label: 'Pizza' ) 
				#(#LabelSpec 
					#layout: #(#Point 61 88 ) 
					#label: 'Toppings' ) 
				#(#LabelSpec 
					#layout: #(#Point 8 168 ) 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #red ) ) 
					#label: 'Entrees' ) 
				#(#LabelSpec 
					#layout: #(#Point 8 232 ) 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #red ) ) 
					#label: 'Deserts' ) 
				#(#LabelSpec 
					#layout: #(#Point 8 312 ) 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #red ) ) 
					#label: 'Beverages' ) 
				#(#LabelSpec 
					#layout: #(#Point 96 392 ) 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #red ) ) 
					#label: 'Total' ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 144 392 296 416 ) 
					#name: #total 
					#model: #total 
					#alignment: #center 
					#isReadOnly: true 
					#type: #fixedpoint ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 200 424 248 448 ) 
					#model: #ok 
					#label: 'OK' 
					#defaultable: true ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 136 88 ) 
					#name: #mozzarella 
					#flags: 32 
					#model: #mozzarella 
					#label: 'Mozzarella' ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 136 112 ) 
					#name: #figs 
					#flags: 32 
					#model: #figs 
					#label: 'Figs' ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 136 136 ) 
					#name: #pineApple 
					#flags: 32 
					#model: #pineApple 
					#label: 'Pineapple' ) 
				#(#LabelSpec 
					#layout: #(#Point 280 8 ) 
					#label: '3.50' ) 
				#(#LabelSpec 
					#layout: #(#Point 280 32 ) 
					#label: '2.50' ) 
				#(#LabelSpec 
					#layout: #(#Point 280 64 ) 
					#label: '3.50' ) 
				#(#LabelSpec 
					#layout: #(#Point 280 88 ) 
					#name: #mozzarellaPrice 
					#flags: 32 
					#label: '1.15' ) 
				#(#LabelSpec 
					#layout: #(#Point 280 112 ) 
					#name: #figsPrice 
					#flags: 32 
					#label: '1.25' ) 
				#(#LabelSpec 
					#layout: #(#Point 280 136 ) 
					#name: #pineapplePrice 
					#flags: 32 
					#label: '1.00' ) 
				#(#LabelSpec 
					#layout: #(#Point 280 168 ) 
					#label: '6.95' ) 
				#(#LabelSpec 
					#layout: #(#Point 280 192 ) 
					#label: '7.25' ) 
				#(#LabelSpec 
					#layout: #(#Point 280 312 ) 
					#label: '0.80' ) 
				#(#LabelSpec 
					#layout: #(#Point 280 336 ) 
					#label: '0.70' ) 
				#(#LabelSpec 
					#layout: #(#Point 280 232 ) 
					#label: '3.70' ) 
				#(#LabelSpec 
					#layout: #(#Point 280 256 ) 
					#label: '4.10' ) 
				#(#LabelSpec 
					#layout: #(#Point 280 280 ) 
					#label: '2.50' ) 
				#(#RegionSpec 
					#layout: #(#Rectangle 328 16 336 448 ) 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #red ) 
						#setBackgroundColor: #(#ColorValue #blue ) ) 
					#lineWidth: 1 ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 96 8 ) 
					#model: #gazpacho 
					#label: 'Gazpacho' ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 96 32 ) 
					#model: #lentil 
					#label: 'Lentil soup' ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 96 168 ) 
					#model: #halibut 
					#label: 'Halibut on Almonds' ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 96 192 ) 
					#model: #chicken 
					#label: 'Chicken Ivanek' ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 96 312 ) 
					#model: #coke 
					#label: 'Coke' ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 96 336 ) 
					#model: #appleJuice 
					#label: 'Apple juice' ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 96 232 ) 
					#model: #applePie 
					#label: 'Jana''s apple pie' ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 96 256 ) 
					#model: #crepes 
					#label: 'Ondrej''s crepes' ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 96 280 ) 
					#model: #brownies 
					#label: 'Dominika''s brownies' ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 96 64 ) 
					#model: #basic 
					#label: 'Basic' ) 
				#(#RegionSpec 
					#layout: #(#Rectangle 8 368 352 376 ) 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #red ) 
						#setBackgroundColor: #(#ColorValue #blue ) ) 
					#lineWidth: 1 ) ) ) )! !

!DriveInMenu class methodsFor: 'opening'!

opening
	"DriveInMenu open"! !

ApplicationModel subclass: #CourseEvalForm
	instanceVariableNames: 'courseNumber questions quest1 quest2 quest3 quest4 quest5 quest6 quest7 quest8 quest9 quest10 quest11 quest12 quest13 comment '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'App 1'!


!CourseEvalForm methodsFor: 'initialization'!

initialize
	quest1 := 'n/a' asValue.
	quest2 := 'n/a' asValue.
	quest3 := 'n/a' asValue.
	quest4 := 'n/a' asValue.
	quest5 := 'n/a' asValue.
	quest6 := 'n/a' asValue.
	quest7 := 'n/a' asValue.
	quest8 := 'n/a' asValue.
	quest9 := 'n/a' asValue.
	quest10 := 'n/a' asValue.
	quest11 := 'n/a' asValue.
	quest12 := 'n/a' asValue.
	quest13 := 'n/a' asValue! !

!CourseEvalForm methodsFor: 'aspects'!

comment
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^comment isNil
		ifTrue:
			[comment := String new asValue]
		ifFalse:
			[comment]!

courseNumber
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^courseNumber isNil
		ifTrue:
			[courseNumber := String new asValue]
		ifFalse:
			[courseNumber]!

quest1
	^quest1!

quest10
	^quest10!

quest11
	^quest11!

quest12
	^quest12!

quest13
	^quest13!

quest2
	^quest2!

quest3
	^quest3!

quest4
	^quest4!

quest5
	^quest5!

quest6
	^quest6!

quest7
	^quest7!

quest8
	^quest8!

quest9
	^quest9! !

!CourseEvalForm methodsFor: 'actions'!

printAndClose
	self gatherQuestions.
	self closeRequest.
	Transcript clear; show: 'Evaluation of course ' , courseNumber value; cr; cr.
	1 to: questions size do: [:index | Transcript show: 'question ' , index printString , ':'; tab; show: (questions at: index); cr]! !

!CourseEvalForm methodsFor: 'private'!

gatherQuestions
	"Calculates array with values for questions"

	questions := Array new: 14.
	1 to: 13
		do: 
			[:index | 
			| value |
			value := (self perform: ('quest' , index printString) asSymbol) value.
			questions at: index put: value asString].
	questions at: 14 put: comment value! !

!CourseEvalForm methodsFor: 'accessing'!

courseNumber: aString 
	courseNumber := aString asValue! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

CourseEvalForm class
	instanceVariableNames: ''!


!CourseEvalForm class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Course Evaluation Form' 
			#min: #(#Point 516 480 ) 
			#max: #(#Point 516 480 ) 
			#bounds: #(#Rectangle 201 69 717 549 ) 
			#colors: 
			#(#LookPreferences 
				#setBackgroundColor: #(#ColorValue 8191 8191 6143 ) ) ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.116279 0 0.0287253 ) 
					#label: 'This is an anonymous evaluation of course ' ) 
				#(#InputFieldSpec 
					#layout: #(#LayoutFrame 0 0.624031 0 0.0287253 0 0.945736 0 0.0718133 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #courseNumber 
					#tabable: false 
					#isReadOnly: false ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.573643 0 0.125673 ) 
					#label: '0     1     2     3     4     5' ) 
				#(#TextEditorSpec 
					#layout: #(#LayoutFrame 0 0.0581395 0 0.70377 0 0.957364 0 0.976661 ) 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #white ) ) 
					#model: #comment ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.0581395 0 0.660682 ) 
					#label: 'Use the space below to enter comments for the instructor:' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.0445736 0 0.152603 ) 
					#label: ' 1. Instructor is well prepared' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.0445736 0 0.18851 ) 
					#label: ' 2. Instructor is enthusiastic' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.0445736 0 0.224417 ) 
					#label: ' 3. Instructor knows the subject well' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.0445736 0 0.29623 ) 
					#label: ' 5. Book is useful' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.0445736 0 0.366248 ) 
					#label: ' 7. Assignment marks are fair' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.0406977 0 0.40395 ) 
					#label: ' 8. Time needed for assignments is reasonable' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.0445736 0 0.439856 ) 
					#label: ' 9. Labs are interesting (if applicable)' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.0445736 0 0.475763 ) 
					#label: '10. Course is stimulating' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.0445736 0 0.51167 ) 
					#label: '11. Overall opinion of course' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.0445736 0 0.552962 ) 
					#label: '12. Would recommend course to others' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.0445736 0 0.590664 ) 
					#label: '13. Course stimulated my interest in this area' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.0445736 0 0.260323 ) 
					#label: ' 4. Instructor can answer questions well' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.0445736 0 0.332136 ) 
					#label: ' 6. Assignments stimulate learning' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.903101 0 0.125673 ) 
					#label: 'N/A' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.567829 0 0.159785 ) 
					#model: #quest1 
					#tabable: false 
					#select: '0' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.624031 0 0.159785 ) 
					#model: #quest1 
					#tabable: false 
					#select: '1' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.728682 0 0.159785 ) 
					#model: #quest1 
					#tabable: false 
					#select: '3' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.676357 0 0.159785 ) 
					#model: #quest1 
					#tabable: false 
					#select: '2' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.781008 0 0.159785 ) 
					#model: #quest1 
					#tabable: false 
					#select: '4' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.833333 0 0.159785 ) 
					#model: #quest1 
					#tabable: false 
					#select: '5' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.918605 0 0.159785 ) 
					#model: #quest1 
					#tabable: false 
					#select: 'n/a' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.567829 0 0.195691 ) 
					#model: #quest2 
					#tabable: false 
					#select: '0' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.624031 0 0.195691 ) 
					#model: #quest2 
					#tabable: false 
					#select: '1' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.728682 0 0.195691 ) 
					#model: #quest2 
					#tabable: false 
					#select: '3' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.676357 0 0.195691 ) 
					#model: #quest2 
					#tabable: false 
					#select: '2' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.781008 0 0.195691 ) 
					#model: #quest2 
					#tabable: false 
					#select: '4' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.833333 0 0.195691 ) 
					#model: #quest2 
					#tabable: false 
					#select: '5' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.918605 0 0.195691 ) 
					#model: #quest2 
					#tabable: false 
					#select: 'n/a' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.567829 0 0.231598 ) 
					#model: #quest3 
					#tabable: false 
					#select: '0' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.624031 0 0.231598 ) 
					#model: #quest3 
					#tabable: false 
					#select: '1' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.728682 0 0.231598 ) 
					#model: #quest3 
					#tabable: false 
					#select: '3' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.676357 0 0.231598 ) 
					#model: #quest3 
					#tabable: false 
					#select: '2' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.781008 0 0.231598 ) 
					#model: #quest3 
					#tabable: false 
					#select: '4' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.833333 0 0.231598 ) 
					#model: #quest3 
					#tabable: false 
					#select: '5' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.918605 0 0.231598 ) 
					#model: #quest3 
					#tabable: false 
					#select: 'n/a' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.567829 0 0.267504 ) 
					#model: #quest4 
					#tabable: false 
					#select: '0' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.624031 0 0.267504 ) 
					#model: #quest4 
					#tabable: false 
					#select: '1' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.728682 0 0.267504 ) 
					#model: #quest4 
					#tabable: false 
					#select: '3' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.676357 0 0.267504 ) 
					#model: #quest4 
					#tabable: false 
					#select: '2' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.781008 0 0.267504 ) 
					#model: #quest4 
					#tabable: false 
					#select: '4' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.833333 0 0.267504 ) 
					#model: #quest4 
					#tabable: false 
					#select: '5' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.918605 0 0.267504 ) 
					#model: #quest4 
					#tabable: false 
					#select: 'n/a' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.567829 0 0.303411 ) 
					#model: #quest5 
					#tabable: false 
					#select: '0' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.624031 0 0.303411 ) 
					#model: #quest5 
					#tabable: false 
					#select: '1' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.728682 0 0.303411 ) 
					#model: #quest5 
					#tabable: false 
					#select: '3' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.676357 0 0.303411 ) 
					#model: #quest5 
					#tabable: false 
					#select: '2' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.781008 0 0.303411 ) 
					#model: #quest5 
					#tabable: false 
					#select: '4' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.833333 0 0.303411 ) 
					#model: #quest5 
					#tabable: false 
					#select: '5' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.918605 0 0.303411 ) 
					#model: #quest5 
					#tabable: false 
					#select: 'n/a' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.567829 0 0.335727 ) 
					#model: #quest6 
					#tabable: false 
					#label: ' ' 
					#select: '0' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.624031 0 0.339318 ) 
					#model: #quest6 
					#tabable: false 
					#select: '1' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.728682 0 0.339318 ) 
					#model: #quest6 
					#tabable: false 
					#select: '3' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.676357 0 0.339318 ) 
					#model: #quest6 
					#tabable: false 
					#select: '2' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.781008 0 0.339318 ) 
					#model: #quest6 
					#tabable: false 
					#select: '4' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.833333 0 0.339318 ) 
					#model: #quest6 
					#tabable: false 
					#select: '5' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.918605 0 0.339318 ) 
					#model: #quest6 
					#tabable: false 
					#select: 'n/a' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.567829 0 0.373429 ) 
					#model: #quest7 
					#tabable: false 
					#select: '0' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.624031 0 0.373429 ) 
					#model: #quest7 
					#tabable: false 
					#select: '1' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.728682 0 0.373429 ) 
					#model: #quest7 
					#tabable: false 
					#select: '3' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.676357 0 0.373429 ) 
					#model: #quest7 
					#tabable: false 
					#select: '2' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.781008 0 0.373429 ) 
					#model: #quest7 
					#tabable: false 
					#select: '4' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.833333 0 0.373429 ) 
					#model: #quest7 
					#tabable: false 
					#select: '5' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.918605 0 0.373429 ) 
					#model: #quest7 
					#tabable: false 
					#select: 'n/a' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.567829 0 0.411131 ) 
					#model: #quest8 
					#tabable: false 
					#select: '0' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.624031 0 0.411131 ) 
					#model: #quest8 
					#tabable: false 
					#select: '1' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.728682 0 0.411131 ) 
					#model: #quest8 
					#tabable: false 
					#select: '3' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.676357 0 0.411131 ) 
					#model: #quest8 
					#tabable: false 
					#select: '2' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.781008 0 0.411131 ) 
					#model: #quest8 
					#tabable: false 
					#select: '4' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.833333 0 0.411131 ) 
					#model: #quest8 
					#tabable: false 
					#select: '5' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.918605 0 0.411131 ) 
					#model: #quest8 
					#tabable: false 
					#select: 'n/a' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.567829 0 0.447038 ) 
					#model: #quest9 
					#tabable: false 
					#select: '0' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.624031 0 0.447038 ) 
					#model: #quest9 
					#tabable: false 
					#select: '1' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.728682 0 0.447038 ) 
					#model: #quest9 
					#tabable: false 
					#select: '3' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.676357 0 0.447038 ) 
					#model: #quest9 
					#tabable: false 
					#select: '2' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.781008 0 0.447038 ) 
					#model: #quest9 
					#tabable: false 
					#select: '4' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.833333 0 0.447038 ) 
					#model: #quest9 
					#tabable: false 
					#select: '5' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.918605 0 0.447038 ) 
					#model: #quest9 
					#tabable: false 
					#select: 'n/a' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.567829 0 0.482944 ) 
					#model: #quest10 
					#tabable: false 
					#select: '0' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.624031 0 0.482944 ) 
					#model: #quest10 
					#tabable: false 
					#select: '1' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.728682 0 0.482944 ) 
					#model: #quest10 
					#tabable: false 
					#select: '3' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.676357 0 0.482944 ) 
					#model: #quest10 
					#tabable: false 
					#select: '2' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.781008 0 0.482944 ) 
					#model: #quest10 
					#tabable: false 
					#select: '4' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.833333 0 0.482944 ) 
					#model: #quest10 
					#tabable: false 
					#select: '5' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.918605 0 0.482944 ) 
					#model: #quest10 
					#tabable: false 
					#select: 'n/a' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.567829 0 0.518851 ) 
					#model: #quest11 
					#tabable: false 
					#select: '0' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.624031 0 0.518851 ) 
					#model: #quest11 
					#tabable: false 
					#select: '1' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.728682 0 0.518851 ) 
					#model: #quest11 
					#tabable: false 
					#select: '3' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.676357 0 0.518851 ) 
					#model: #quest11 
					#tabable: false 
					#select: '2' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.781008 0 0.518851 ) 
					#model: #quest11 
					#tabable: false 
					#select: '4' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.833333 0 0.518851 ) 
					#model: #quest11 
					#tabable: false 
					#select: '5' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.918605 0 0.518851 ) 
					#model: #quest11 
					#tabable: false 
					#select: 'n/a' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.567829 0 0.560144 ) 
					#model: #quest12 
					#tabable: false 
					#select: '0' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.624031 0 0.560144 ) 
					#model: #quest12 
					#tabable: false 
					#select: '1' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.728682 0 0.560144 ) 
					#model: #quest12 
					#tabable: false 
					#select: '3' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.676357 0 0.560144 ) 
					#model: #quest12 
					#tabable: false 
					#select: '2' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.781008 0 0.560144 ) 
					#model: #quest12 
					#tabable: false 
					#select: '4' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.833333 0 0.560144 ) 
					#model: #quest12 
					#tabable: false 
					#select: '5' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.918605 0 0.560144 ) 
					#model: #quest12 
					#tabable: false 
					#select: 'n/a' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.567829 0 0.597846 ) 
					#model: #quest13 
					#tabable: false 
					#select: '0' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.624031 0 0.597846 ) 
					#model: #quest13 
					#tabable: false 
					#select: '1' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.728682 0 0.597846 ) 
					#model: #quest13 
					#tabable: false 
					#select: '3' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.676357 0 0.597846 ) 
					#model: #quest13 
					#tabable: false 
					#select: '2' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.781008 0 0.597846 ) 
					#model: #quest13 
					#tabable: false 
					#select: '4' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.833333 0 0.597846 ) 
					#model: #quest13 
					#tabable: false 
					#select: '5' ) 
				#(#RadioButtonSpec 
					#layout: #(#LayoutOrigin 0 0.918605 0 0.597846 ) 
					#model: #quest13 
					#tabable: false 
					#select: 'n/a' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.76938 0 0.653501 0 0.988372 0 0.696589 ) 
					#model: #printAndClose 
					#label: 'Print and close' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.0445736 0 0.0861759 ) 
					#label: 'For each question, select a rating between 0 (worst) and 5 (best)' ) ) ) )! !

!CourseEvalForm class methodsFor: 'opening'!

opening
	"CourseEvalForm openOnCourse: 'COMP 1213'"!

openOnCourse: aString 
	| form |
	form := self new.
	form courseNumber: aString.
	self openOn: form! !

