Object subclass: #Piece
	instanceVariableNames: 'color position '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'App 3'!


!Piece methodsFor: 'accessing'!

color
	^color!

color: aSymbol 
	color := aSymbol!

position
	^position!

position: aPoint 
	position := aPoint! !

!Piece methodsFor: 'testing'!

isKing
	^false!

legalMove: kindOfMove to: aPoint
	"Return true if this piece can move to this destination."

	self subclassResponsibility! !

!Piece methodsFor: 'displaying'!

imageForPlayer: symbol1 onBackground: symbol2 
	^(self class imageForPlayer: symbol1 onBackground: symbol2) asImage! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Piece class
	instanceVariableNames: 'WhiteOnWhite WhiteOnBlack BlackOnWhite BlackOnBlack '!


!Piece class methodsFor: 'accessing'!

color: symbol1 onColor: symbol2 
	"Retrieve the appropriate image."

	symbol1 == #white
		ifTrue: [symbol2 == #white
				ifTrue: [^WhiteOnWhite]
				ifFalse: [^WhiteOnBlack]]
		ifFalse: [symbol2 == #white
				ifTrue: [^BlackOnWhite]
				ifFalse: [^BlackOnBlack]]! !

!Piece class methodsFor: 'initialization'!

initialization
	"Assign images to class instance variables."

	self subclassResponsibility! !

!Piece class methodsFor: 'creation'!

color: aSymbol position: aPoint 
	^(self new) color: aSymbol; position: aPoint! !

!Piece class methodsFor: 'images'!

blackOnBlack
	self subclassResponsibility!

blackOnWhite
	self subclassResponsibility!

whiteOnBlack
	self subclassResponsibility!

whiteOnWhite
	self subclassResponsibility! !

!Piece class methodsFor: 'image'!

imageForPlayer: symbol1 onBackground: symbol2 
^self perform: (symbol1 == #white
			ifTrue: [symbol2 == #white
					ifTrue: [#whiteOnWhite]
					ifFalse: [#whiteOnBlack]]
			ifFalse: [symbol2 == #white
					ifTrue: [#blackOnWhite]
					ifFalse: [#blackOnBlack]])! !

Piece subclass: #Queen
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'App 3'!


!Queen methodsFor: 'testing'!

legalMove: kindOfMove to: aPoint
	"Can this piece go to this destination?"

	^kindOfMove ~~ #illegal! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Queen class
	instanceVariableNames: ''!


!Queen class methodsFor: 'resources'!

blackOnBlack
	"UIMaskEditor new openOnClass: self andSelector: #blackOnBlack"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue black; at: 2 put: ColorValue lightGray; at: 3 put: ColorValue white; yourself)) usingBits: (ByteArray fromPackedString: 'UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUPEUUUUUUUUUHUUUUUUUUAT!!TEUUUUUTARE@UUUUUUPEHTAUUUUUUPD!!@UUUUUUP@RD@EUUUUTB@@@ EUUUUTB@@HAUUUUUTB@B@UUUUUUTB@ EUUUUUUTHBAUUUUUUT@@@@UUUUUUP@@@AUUUUUU@B(@EUUUUUT@@@@UUUUUUP@@@AUUUUUU@(B EUUUUUT@@@@UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUP@a'))!

blackOnWhite
	"UIMaskEditor new openOnClass: self andSelector: #blackOnWhite"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 3 128 0 0 2 128 0 0 98 140 0 0 114 156 0 0 114 156 0 0 58 184 0 0 250 190 0 1 223 247 0 0 239 238 0 0 119 220 0 0 59 184 0 0 27 176 0 0 127 252 0 0 127 252 0 0 124 124 0 0 127 252 0 0 127 252 0 0 115 156 0 0 127 252 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0])!

whiteOnBlack
	"UIMaskEditor new openOnClass: self andSelector: #whiteOnBlack"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue black; at: 2 put: ColorValue lightGray; at: 3 put: ColorValue white; yourself)) usingBits: (ByteArray fromPackedString: 'UUUUUUUUUZ)UUUUUUUUU*%UUUUUUUUV*UUUUUUUUUZ)UUUUUUUUU*%UUUUUUUUV*UUUUUUUUUZ)UUUUPEUUU*%UUUUHUUUV*UUUAT!!TEUZ)UUT!!REHUU*%UUREHT!!UV*UUURD!!HUUZ)UUPHRD EU*%UTHH*HHEV*UUTHJ*BAUZ)UUTH*( UU*%UUTJ*(EUV*UUUTJ*AUUZ)UUT@@@@UU*%UUR***!!UV*UUUJ***EUZ)UUT@@@@UU*%UUR***!!UV*UUUJ***EUZ)UUT@@@@UU*%UUUUUUUUV*UUUUUUUUUZ)UUUUUUUUU********************************* @a'))!

whiteOnWhite
	"UIMaskEditor new openOnClass: self andSelector: #whiteOnWhite"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 3 128 0 0 2 128 0 0 98 140 0 0 82 148 0 0 82 148 0 0 42 168 0 0 234 174 0 1 180 91 0 0 216 54 0 0 104 44 0 0 48 24 0 0 24 48 0 0 127 252 0 0 64 4 0 0 64 4 0 0 127 252 0 0 64 4 0 0 64 4 0 0 127 252 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0])! !

Piece subclass: #Pawn
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'App 3'!


!Pawn methodsFor: 'testing'!

legalMove: kindOfMove to: aPoint 
	"Can this piece go to this destination?"

	kindOfMove == #vertical ifTrue: [color == #white
			ifTrue: [^aPoint y - position y = -1 or: [position y = 7 and: [aPoint y - position y = -2]]]
			ifFalse: [^aPoint y - position y = 1 or: [position y = 2 and: [aPoint y - position y = 2]]]].
	^false! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Pawn class
	instanceVariableNames: ''!


!Pawn class methodsFor: 'resources'!

blackOnBlack
	"UIMaskEditor new openOnClass: self andSelector: #blackOnBlack"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue black; at: 2 put: ColorValue lightGray; at: 3 put: ColorValue white; yourself)) usingBits: (ByteArray fromPackedString: 'UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU@AUUUUUUUUR*!!UUUUUUUT(B!!UUUUUUUR@BEUUUUUUUJ@HUUUUUUUUJ*EUUUUUUUU@AUUUUUUUUP@AUUUUUUUT@@AUUUUUUUH@@!!UUUUUUT @BEUUUUUUT@@AUUUUUUUT*(UUUUUUUU@@@UUUUUUUR@@HUUUUUUT @@HUUUUUUH@@@HUUUUURB**(HUUUUT@@@@@@UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUP@a'))!

blackOnWhite
	"UIMaskEditor new openOnClass: self andSelector: #blackOnWhite"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 7 192 0 0 8 32 0 0 19 144 0 0 23 208 0 0 19 208 0 0 8 32 0 0 7 192 0 0 15 224 0 0 31 240 0 0 47 232 0 0 47 232 0 0 31 240 0 0 8 32 0 0 31 240 0 0 47 232 0 0 95 244 0 0 191 250 0 1 96 13 0 3 255 255 128 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0])!

whiteOnBlack
	"UIMaskEditor new openOnClass: self andSelector: #whiteOnBlack"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue black; at: 2 put: ColorValue lightGray; at: 3 put: ColorValue white; yourself)) usingBits: (ByteArray fromPackedString: 'UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU@AUUUUUUUUR*!!UUUUUUUT**!!UUUUUUUR**EUUUUUUUJ*(UUUUUUUUJ*EUUUUUUUUJ!!UUUUUUUUP@AUUUUUUUT**!!UUUUUUUJ**!!UUUUUUT***EUUUUUUT**!!UUUUUUUT@@UUUUUUUUJ*(UUUUUUUR**(UUUUUUT***(UUUUUUJ***(UUUUUR****(UUUUT@@@@@@UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUP@a'))!

whiteOnWhite
	"UIMaskEditor new openOnClass: self andSelector: #whiteOnWhite"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 3 224 0 0 4 16 0 0 8 8 0 0 8 8 0 0 8 8 0 0 4 16 0 0 2 32 0 0 7 240 0 0 8 8 0 0 16 4 0 0 16 4 0 0 8 8 0 0 7 240 0 0 8 8 0 0 16 4 0 0 32 2 0 0 64 1 0 0 128 0 128 1 255 255 192 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0])! !

View subclass: #ChessView
	instanceVariableNames: 'squareExtent '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'App 3'!


!ChessView methodsFor: 'default controller'!

defaultControllerClass
	^ChessController! !

!ChessView methodsFor: 'updating'!

update: aSymbol with: anArray 
	| square |
	1 to: 2
		do: 
			[:i | 
			square := anArray at: i.self invalidateRectangle: (square - 1 * squareExtent extent: squareExtent)
				repairNow: true]! !

!ChessView methodsFor: 'displaying'!

displayOn: aGraphicsContext 
	squareExtent := self squareExtent.
	1 to: 8 do: [:row | 1 to: 8
			do: 
				[:column | 
				| background piece |
				background := (row + column) even
							ifTrue: [#white]
							ifFalse: [#lightGray].
				piece := self model squares at: column @ row.
				piece isNil
					ifTrue: [aGraphicsContext paint: (ColorValue perform: background); displayRectangle: (0 @ 0 extent: squareExtent @ squareExtent)
							at: column - 1 * squareExtent @ (row - 1 * squareExtent)]
					ifFalse: [aGraphicsContext displayImage: (piece imageForPlayer: piece color onBackground: background)
							at: column - 1 * squareExtent @ (row - 1 * squareExtent)]]]! !

!ChessView methodsFor: 'accessing'!

squareExtent
	^squareExtent isNil
		ifTrue: [squareExtent := self container bounds extent x / 8]
		ifFalse: [squareExtent]! !

Piece subclass: #King
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'App 3'!


!King methodsFor: 'testing'!

isKing
	^true!

legalMove: kindOfMove to: aPoint
	^(position x - aPoint x) abs < 2 and: [(position y - aPoint y) abs < 2]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

King class
	instanceVariableNames: ''!


!King class methodsFor: 'resources'!

blackOnBlack
	"UIMaskEditor new openOnClass: self andSelector: #blackOnBlack"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue black; at: 2 put: ColorValue lightGray; at: 3 put: ColorValue white; yourself)) usingBits: (ByteArray fromPackedString: 'UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUTAPE@UUUUUUJ!!HR(UUUUURB@  (UUUUUHB"H@!!UUUUT BH BEUUUUT H @!!UUUUUR@@@BEUUUUUR@@@!!UUUUUUH@@BEUUUUUUH@@!!UUUUUUT(@JEUUUUUUT @!!UUUUUUUR**EUUUUUUU@@@UUUUUUUT@@AUUUUUUUP@@EUUUUUUU@@@UUUUUUUT**!!UUUUUUUP@@EUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUP@a'))!

blackOnWhite
	"UIMaskEditor new openOnClass: self andSelector: #blackOnWhite"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 115 156 0 0 138 162 0 1 110 217 0 1 114 189 0 1 122 189 0 0 186 250 0 0 191 250 0 0 95 244 0 0 95 244 0 0 47 232 0 0 39 200 0 0 23 208 0 0 16 16 0 0 31 240 0 0 31 240 0 0 31 240 0 0 31 240 0 0 16 16 0 0 31 240 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0])!

whiteOnBlack
	"UIMaskEditor new openOnClass: self andSelector: #whiteOnBlack"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue black; at: 2 put: ColorValue lightGray; at: 3 put: ColorValue white; yourself)) usingBits: (ByteArray fromPackedString: 'UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUTAPE@UUUUUUJ!!HR(UUUUUR*  *(UUUUUJ*"J*!!UUUUT**H**EUUUUT*("*!!UUUUUR* J*EUUUUUR***!!UUUUUUJ***EUUUUUUJ**!!UUUUUUT***EUUUUUUT**!!UUUUUUUP@@EUUUUUUUJ*(UUUUUUUT**!!UUUUUUUR**EUUUUUUUJ*(UUUUUUUT**!!UUUUUUUP@@EUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUP@a'))!

whiteOnWhite
	"UIMaskEditor new openOnClass: self andSelector: #whiteOnWhite"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 115 156 0 0 138 162 0 1 6 193 0 1 2 129 0 1 2 129 0 0 130 130 0 0 131 130 0 0 64 4 0 0 64 4 0 0 32 8 0 0 32 8 0 0 16 16 0 0 31 240 0 0 16 16 0 0 16 16 0 0 16 16 0 0 16 16 0 0 16 16 0 0 31 240 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0])! !

Piece subclass: #Bishop
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'App 3'!


!Bishop methodsFor: 'testing'!

legalMove: kindOfMove to: aPoint
	"Can this piece go to this destination?"

	^kindOfMove == #diagonal! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Bishop class
	instanceVariableNames: ''!


!Bishop class methodsFor: 'resources'!

blackOnBlack
	"UIMaskEditor new openOnClass: self andSelector: #blackOnBlack"

	<resource: #image>
	^CachedImage on: (Image extent: 31@31 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue black; at: 2 put: ColorValue lightGray; at: 3 put: ColorValue white; yourself)) usingBits: (ByteArray fromPackedString: 'UUUUUUUUUUQUUUUUUUUUUEUUUUUUUUUTUUUUUUUUUUQUUUUUUUUUUEUUUUUUUUUTUUUUUEUUUUQUUUUREUUUUEUUUQHQUUUTUUUT"HAUUUQUUUBBHAUUUEUUP BHAUUTUUUB@B@EUUQUUTHBB@UUUEUUTHHBEUUTUUUP  HUUUQUUU@"@EUUUEUUUR@AUUUTUUUUJ*EUUUQUUU@@@AUUUEUUT@@@EUUTUUUP@@@UUUQUUU@@@AUUUEUP@@@@@AUTUUJ*!!T**EUQUT@@EP@@UUEUUUUUUUUUTUUUUUUUUUUQUUUUUUUUUUEUUUUUUUUUTUUUUUUUUUUPb'))!

blackOnWhite
	"UIMaskEditor new openOnClass: self andSelector: #blackOnWhite"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 2 128 0 0 10 160 0 0 21 112 0 0 54 184 0 0 111 92 0 0 111 188 0 0 110 220 0 0 54 232 0 0 54 232 0 0 58 248 0 0 11 224 0 0 8 32 0 0 63 248 0 0 63 248 0 0 63 248 0 0 63 248 0 3 255 255 128 2 4 64 128 3 252 127 128 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0])!

whiteOnBlack
	"UIMaskEditor new openOnClass: self andSelector: #whiteOnBlack"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue black; at: 2 put: ColorValue lightGray; at: 3 put: ColorValue white; yourself)) usingBits: (ByteArray fromPackedString: 'UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUEUUUUUUUUUREUUUUUUUUQHQUUUUUUUT"H!!UUUUUUUJ"J!!UUUUUURJ"(!!UUUUUUH*""EUUUUUT"**HUUUUUUT"*"EUUUUUURJ*HUUUUUUUBJ"EUUUUUUUR*!!UUUUUUUUJ*EUUUUUUU@@@AUUUUUUT***EUUUUUUR**(UUUUUUUJ**!!UUUUUP@@@@@AUUUUJ*!!T**EUUUT@@EP@@UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUP@a'))!

whiteOnWhite
	"UIMaskEditor new openOnClass: self andSelector: #whiteOnWhite"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 2 128 0 0 10 160 0 0 21 80 0 0 34 136 0 0 81 20 0 0 80 148 0 0 80 20 0 0 40 40 0 0 40 40 0 0 20 80 0 0 8 32 0 0 8 32 0 0 63 248 0 0 32 8 0 0 32 8 0 0 32 8 0 3 255 255 128 2 4 64 128 3 252 127 128 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0])! !

Model subclass: #ChessBoard
	instanceVariableNames: 'squares kings kindOfMove blackPieces whitePieces currentPlayer oldPosition newPosition removedPiece appModel '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'App 3'!
ChessBoard comment:
'Domain model of chess board. White starts at bottom, black on top, board coordinates  are points with x = column, y = row. Knows about pieces, knows how to make moves (in collaboration with pieces).

squares		<TwoDList>			8x8 squares containing nil or a Piece
kings 		<Array>					first element is white King, second is black King
kindOfMove	<#horizontal, #vertical, #diagonal, #illegal>	#illegal may be OK for Knight
blackPieces	<OrderedCollection>		collection of currently playing black pieces
whitePieces <OrderedCollection>	collection of currently playing black pieces
currentPlayer<#white or #black>		color of player about to move
oldPosition	<Point>					starting position of piece attempting to move
newPosition	<Point>					destination position of piece attempting to move
removedPiece	<Piece>			opponent''s piece removed by move, needed if move must be retracted'!


!ChessBoard methodsFor: 'initialization'!

initializeFrom: app 
	"Initialize board position and instance variables."
	"Create set of white and black pieces, array with kings."
	"ChessBoard new initialize"

	appModel := app.
	self newGame!

initializePieces
	whitePieces := OrderedCollection new: 16.
	blackPieces := OrderedCollection new: 16.
	kings := Array new: 2.
	squares do: [:piece | piece isNil ifFalse: [piece color == #white
				ifTrue: 
					[whitePieces add: piece.
					piece isKing ifTrue: [kings at: 1 put: piece]]
				ifFalse: 
					[blackPieces add: piece.
					piece isKing ifTrue: [kings at: 2 put: piece]]]]!

initializeSquares
	| pos |
	squares := TwoDList columns: 8 rows: 8.
	1 to: 8 do: [:col | "Blank empty squares."
		1 to: 8 do: [:row | squares at: row @ col put: nil]].
	1 to: 8
		do: 
			[:col | 
			"Place remaining pieces."
			"Place pawns."
			squares at: (pos := col @ 2) put: (Pawn color: #black position: pos).
			squares at: (pos := col @ 7) put: (Pawn color: #white position: pos)].	"Place all the rest."
	squares at: (pos := 1 @ 1) put: (Rook color: #black position: pos).
	squares at: (pos := 1 @ 8) put: (Rook color: #white position: pos).
	squares at: (pos := 8 @ 1) put: (Rook color: #black position: pos).
	squares at: (pos := 8 @ 8) put: (Rook color: #white position: pos).
	squares at: (pos := 2 @ 1) put: (Knight color: #black position: pos).
	squares at: (pos := 2 @ 8) put: (Knight color: #white position: pos).
	squares at: (pos := 7 @ 1) put: (Knight color: #black position: pos).
	squares at: (pos := 7 @ 8) put: (Knight color: #white position: pos).
	squares at: (pos := 3 @ 1) put: (Bishop color: #black position: pos).
	squares at: (pos := 3 @ 8) put: (Bishop color: #white position: pos).
	squares at: (pos := 6 @ 1) put: (Bishop color: #black position: pos).
	squares at: (pos := 6 @ 8) put: (Bishop color: #white position: pos).
	squares at: (pos := 4 @ 1) put: (Queen color: #black position: pos).
	squares at: (pos := 4 @ 8) put: (Queen color: #white position: pos).
	squares at: (pos := 5 @ 1) put: (King color: #black position: pos).
	squares at: (pos := 5 @ 8) put: (King color: #white position: pos)!

newGame
	self initializeSquares.
	self initializePieces.
	currentPlayer := #white! !

!ChessBoard methodsFor: 'private'!

diagonalClearFrom: point1 to: point2 
	"Is the diagonal stretch between the two points clear?"

	| x1 y1 numberOfSquares stepx stepy x y |
	x1 := point1 x.
	y1 := point1 y.
	(numberOfSquares := (point2 y - y1) abs - 1) = 0 ifTrue: [^true].
	x1 > point2 x
		ifTrue: [stepx := -1]
		ifFalse: [stepx := 1].
	y1 > point2 y
		ifTrue: [stepy := -1]
		ifFalse: [stepy := 1].
	x := x1.
	y := y1.
	numberOfSquares timesRepeat: [(squares at: (x := x + stepx) @ (y := y + stepy)) isNil ifFalse: [^false]].
	^true!

horizontalClearFrom: point1 to: point2 
	"Is the horizontal stretch between the two points clear?"


	| x1 x2 y step |
	x1 := point1 x.
	x2 := point2 x.
	y := point1 y.
	x1 > x2
		ifTrue: [step := -1]
		ifFalse: [step := 1].
	x1 + step to: x2 - step by: step do: [:x | (squares at: x @ y) isNil ifFalse: [^false]].
	^true!

kindOfMoveFrom: point1 to: point2
"Is the move horizontal, vertical, diagonal, or illegal? Knight overrides."

	kindOfMove := point1 y = point2 y
				ifTrue: [#horizontal]
				ifFalse: [point1 x = point2 x
						ifTrue: [#vertical]
						ifFalse: [(point1 x - point2 x) abs = (point1 y - point2 y) abs
								ifTrue: [#diagonal]
								ifFalse: [#illegal]]].!

piece: aPiece canReach: point2 from: point1 
	"Can aPiece go from point1 to point2?"

	self kindOfMoveFrom: point1 to: point2.
	(aPiece legalMove: kindOfMove to: point2)
		ifFalse: [^false].
	^self isClearFrom: point1 to: point2!

switchPlayer
	currentPlayer == #white
		ifTrue: [currentPlayer := #black]
		ifFalse: [currentPlayer := #white].
	appModel player: currentPlayer!

verticalClearFrom: point1 to: point2 
	"Is the vertical stretch between the two points clear?"
	| x y1 y2 step |
	x := point1 x.
	y1 := point1 y.
	y2 := point2 y.
	y1 > y2
		ifTrue: [step := -1]
		ifFalse: [step := 1].
	y1 + step to: y2 - step by: step do: [:y | (squares at: x @ y) isNil ifFalse: [^false]].
	^true! !

!ChessBoard methodsFor: 'testing'!

checks
	"Test whether the king of the opponent is in check."

	| position |
	position := self opponentKing position.
	self myPieces do: [:piece | (self
			piece: piece
			canReach: position
			from: piece position)
			ifTrue: [^true]].
	^false!

inCheck
	"Test whether the king of the current player is in check."

	| position |
	position := self myKing position.
	self opponentPieces do: [:piece | (self
			piece: piece
			canReach: position
			from: piece position)
			ifTrue: [^true]].
	^false!

isClearFrom: point1 to: point2 
	"Is the path from point1 to point2 clear?"

	kindOfMove == #horizontal ifTrue: [^self horizontalClearFrom: point1 to: point2].
	kindOfMove == #vertical ifTrue: [^self verticalClearFrom: point1 to: point2].
	kindOfMove == #diagonal ifTrue: [^self diagonalClearFrom: point1 to: point2].	
	^true "for knight"!

isOccupied: aPoint 
	"Is this destination square occupied by my own piece?"

	| piece |
	piece := squares at: aPoint.
	^(piece isNil or: [piece color ~~ currentPlayer]) not!

isSquareLegal: aPoint 
	"Is this square a legal starting point for a move?"

	| piece |
	piece := self pieceAt: aPoint.
	^piece isNil not and: [piece color == currentPlayer]! !

!ChessBoard methodsFor: 'accessing'!

blackPieces
	^blackPieces!

currentPlayer
	^currentPlayer!

currentPlayer: aSymbol 
	currentPlayer := aSymbol!

myKing
	^currentPlayer == #white
		ifTrue: [kings at: 1]
		ifFalse: [kings at: 2]!

myPieces
	currentPlayer == #white
		ifTrue: [^whitePieces]
		ifFalse: [^blackPieces]!

newPosition: aPoint 
	newPosition := aPoint!

oldPosition: aPoint 
	oldPosition := aPoint!

opponentKing
	^currentPlayer == #white
		ifTrue: [kings at: 2]
		ifFalse: [kings at: 1]!

opponentPieces
	currentPlayer == #white
		ifTrue: [^blackPieces]
		ifFalse: [^whitePieces]!

pieceAt: aPoint 
	^squares at: aPoint!

squares
	^squares!

whitePieces
	^whitePieces! !

!ChessBoard methodsFor: 'moving'!

move
	"Return true if move is legal, false otherwise. Make move if legal."

	| piece destPiece |
	piece := squares at: oldPosition.
	((destPiece := squares at: newPosition) isNil or: [destPiece color ~~ currentPlayer])
		ifFalse: [^false].
	(self
		piece: piece
		canReach: newPosition
		from: oldPosition)
		ifFalse: [^false].
	self move: piece to: newPosition.
	self inCheck
		ifTrue: 
			[self restore: piece.
			^false].
	(appModel builder componentAt: #check) beInvisible.
	self  changed: #position with: (Array with: oldPosition with: newPosition).
	self checks ifTrue: [(appModel builder componentAt: #check) beVisible].
	self switchPlayer.
	^true!

move: aPiece to: aPoint 
	"Empty starting position, put piece in new position, remove piece at destination position if any."

	squares at: oldPosition put: nil.
	(removedPiece := squares at: aPoint) isNil ifFalse: [self opponentPieces remove: removedPiece].
	squares at: aPoint put: aPiece.
	aPiece position: aPoint!

restore: aPiece 
	"Attempted move illegal - exposes king to check. Restore original state of board."

	squares at: oldPosition put: aPiece.
	removedPiece isNil ifFalse: [self opponentPieces add: aPiece].
	squares at: aPiece position put: removedPiece.
	aPiece position: oldPosition! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ChessBoard class
	instanceVariableNames: ''!


!ChessBoard class methodsFor: 'creation'!

newFrom: appModel 
	^self new initializeFrom: appModel! !

Piece subclass: #Rook
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'App 3'!


!Rook methodsFor: 'testing'!

legalMove: kindOfMove to: aPoint
	"Can this piece go to this destination?"

	^kindOfMove == #horizontal or: [kindOfMove == #vertical]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Rook class
	instanceVariableNames: ''!


!Rook class methodsFor: 'resources'!

blackOnBlack
	"UIMaskEditor new openOnClass: self andSelector: #blackOnBlack"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue black; at: 2 put: ColorValue lightGray; at: 3 put: ColorValue white; yourself)) usingBits: (ByteArray fromPackedString: 'UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUTAPUAPEUUUUP@@@@@UUUUU@(J (AUUUUT@@@@@EUUUUP@@@@@UUUUUPJ** EUUUUUT@ @EUUUUUUPB@@UUUUUUUJ**!!UUUUUUT@B@EUUUUUUP@H@UUUUUUUJ**!!UUUUUUT@ @EUUUUUUPB@@UUUUUUUJ**!!UUUUUUP@@@AUUUUUT@@@@AUUUUT@J** @UUUUP@@@@@AUUUU@@@@@@EUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUP@a'))!

blackOnWhite
	"UIMaskEditor new openOnClass: self andSelector: #blackOnWhite"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 230 51 128 0 255 255 128 0 230 51 128 0 255 255 128 0 255 255 128 0 112 7 0 0 30 252 0 0 30 252 0 0 16 4 0 0 31 188 0 0 31 188 0 0 16 4 0 0 30 252 0 0 30 252 0 0 16 4 0 0 63 254 0 0 127 255 0 1 240 7 192 1 255 255 192 1 255 255 192 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0])!

whiteOnBlack
	"UIMaskEditor new openOnClass: self andSelector: #whiteOnBlack"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue black; at: 2 put: ColorValue lightGray; at: 3 put: ColorValue white; yourself)) usingBits: (ByteArray fromPackedString: 'UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUTAPUAPEUUUUR (B (UUUUUJ****!!UUUUT*****EUUUUP**** UUUUUP@@@@EUUUUUT*J*EUUUUUUR(*(UUUUUUU@@@AUUUUUUT*(*EUUUUUUR*"(UUUUUUU@@@AUUUUUUT*J*EUUUUUUR(*(UUUUUUU@@@AUUUUUUP***AUUUUUT****!!UUUUTB @@J@UUUUR*****!!UUUU@@@@@@EUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUP@a'))!

whiteOnWhite
	"UIMaskEditor new openOnClass: self andSelector: #whiteOnWhite"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 230 51 128 0 153 204 128 0 128 0 128 0 128 0 128 0 192 1 128 0 127 255 0 0 17 4 0 0 17 4 0 0 31 252 0 0 16 68 0 0 16 68 0 0 31 252 0 0 17 4 0 0 17 4 0 0 31 252 0 0 48 6 0 0 64 1 0 1 207 249 192 1 0 0 64 1 255 255 192 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0])! !

Piece subclass: #Knight
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'App 3'!


!Knight methodsFor: 'testing'!

legalMove: kindOfMove to: aPoint
	"Can this piece go to this destination?"

	kindOfMove == #illegal ifTrue: [^((position x - aPoint x) abs = 2 and: [(position y - aPoint y) abs = 1])
			or: [(position x - aPoint x) abs = 1 and: [(position y - aPoint y) abs = 2]]].
	^false! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Knight class
	instanceVariableNames: ''!


!Knight class methodsFor: 'resources'!

blackOnBlack
	"UIMaskEditor new openOnClass: self andSelector: #blackOnBlack"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue black; at: 2 put: ColorValue lightGray; at: 3 put: ColorValue white; yourself)) usingBits: (ByteArray fromPackedString: 'UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUDUUUUUUUUUP@UUUUUUUUU@@AUUUUUUUP@@AUUUUUUU@@*AUUUUUUPH@BAUUUUUU@(@BAUUUUUP@@HBAUUUUT@@B@HEUUUUB@@(@ UUUUTH@@*B@UUUUP@EP@BAUUUUPEUP@HEUUUUUUU@@ UUUUUUU@@BAUUUUUU@@@HEUUUUUP@@@ UUUUUTJ**(AUUUUUP@@@@UUUUUU@@@@EUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUP@a'))!

blackOnWhite
	"UIMaskEditor new openOnClass: self andSelector: #blackOnWhite"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 5 0 0 0 15 128 0 0 15 240 0 0 31 248 0 0 31 140 0 0 59 246 0 0 57 251 0 0 127 221 128 0 255 189 128 1 191 61 128 1 191 141 192 1 248 254 192 0 224 126 192 0 0 126 192 0 1 254 192 0 7 254 192 0 15 254 192 0 24 1 192 0 31 255 128 0 31 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0])!

whiteOnBlack
	"UIMaskEditor new openOnClass: self andSelector: #whiteOnBlack"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue black; at: 2 put: ColorValue lightGray; at: 3 put: ColorValue white; yourself)) usingBits: (ByteArray fromPackedString: 'UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUQEUUUUUUUUT"EUUUUUUUURH@UUUUUUUT**(UUUUUUUR**(UUUUUUT(**(UUUUUUR **(UUUUUT**(*HUUUUUJ**J(!!UUUUR**"*"EUUUUJ*@@JJEUUUT*AT**HUUUUTAUT*(!!UUUUUUUR*"EUUUUUUPJ*HUUUUUUP**(!!UUUUUT***"EUUUUUJ@@@(UUUUUT****EUUUUUP@@@AUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUP@a'))!

whiteOnWhite
	"UIMaskEditor new openOnClass: self andSelector: #whiteOnWhite"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 5 0 0 0 10 128 0 0 10 240 0 0 16 8 0 0 16 4 0 0 36 2 0 0 38 1 0 0 64 34 128 0 128 66 128 1 0 130 128 1 7 242 64 1 24 129 64 0 224 65 64 0 0 65 64 0 1 193 64 0 6 1 64 0 8 1 64 0 19 254 64 0 16 0 128 0 31 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0])! !

Controller subclass: #ChessController
	instanceVariableNames: 'firstClick oldCursor currentCursor blackCursor destCursor whiteCursor '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'App 3'!


!ChessController methodsFor: 'events'!

enterEvent: event 
	oldCursor := Cursor currentCursor.
	Cursor currentCursor: currentCursor.
	currentCursor show!

exitEvent: event 
	Cursor currentCursor: oldCursor.
	Cursor normal show!

redButtonPressedEvent: event 
	| square |
	(self viewHasCursorWithEvent: event)
		ifFalse: [^self].
	square := ((self sensor cursorPointFor: event)
				/ self view squareExtent) truncated + (1 @ 1).
	firstClick
		ifTrue: 
			[(self model isSquareLegal: square)
				ifFalse: [^self].
			self model oldPosition: square.
			firstClick := false.
			currentCursor := destCursor]
		ifFalse: 
			[self model newPosition: square.
			self model move.
			firstClick := true.
			currentCursor := self model currentPlayer == #white
						ifTrue: [whiteCursor]
						ifFalse: [blackCursor]].
	currentCursor show! !

!ChessController methodsFor: 'initialization'!

initialize
	firstClick := true.
	whiteCursor := Cursor
				image: ChessController aWhiteCursor asImage
				mask: ChessController aWhiteCursorMask asImage
				hotSpot: 4 @ 8
				name: 'white moves'.
	blackCursor := Cursor
				image: ChessController aBlackCursor asImage
				mask: ChessController aBlackCursorMask asImage
				hotSpot: 4 @ 8
				name: 'black moves'.
	destCursor := Cursor
				image: ChessController aDestCursor asImage
				mask: ChessController aDestCursorMask asImage
				hotSpot: 4 @ 8
				name: 'destination'.
	currentCursor := whiteCursor! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ChessController class
	instanceVariableNames: ''!


!ChessController class methodsFor: 'resources'!

aBlackCursor
	"UIMaskEditor new openOnClass: self andSelector: #aBlackCursor"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[0 0 0 0 31 192 0 0 16 32 0 0 16 16 0 0 16 16 0 0 16 16 0 0 16 32 0 0 31 192 0 0 16 32 0 0 16 16 0 0 16 8 0 0 16 8 0 0 16 8 0 0 16 16 0 0 31 224 0 0 0 0 0 0])!

aBlackCursorMask
	"UIMaskEditor new openOnClass: self andSelector: #aBlackCursorMask"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 1 bitsPerPixel: 1 palette: CoveragePalette monoMaskPalette usingBits: #[63 192 0 0 63 224 0 0 63 240 0 0 56 56 0 0 56 56 0 0 56 56 0 0 63 240 0 0 63 224 0 0 63 240 0 0 56 56 0 0 56 28 0 0 56 28 0 0 56 28 0 0 63 248 0 0 63 240 0 0 63 240 0 0])!

aDestCursor
	"UIMaskEditor new openOnClass: self andSelector: #aDestCursor"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[0 0 0 0 31 192 0 0 16 32 0 0 16 16 0 0 16 16 0 0 16 16 0 0 16 16 0 0 16 16 0 0 16 16 0 0 16 16 0 0 16 16 0 0 16 16 0 0 16 16 0 0 16 32 0 0 31 192 0 0 0 0 0 0])!

aDestCursorMask
	"UIMaskEditor new openOnClass: self andSelector: #aDestCursorMask"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 1 bitsPerPixel: 1 palette: CoveragePalette monoMaskPalette usingBits: #[63 224 0 0 63 224 0 0 63 240 0 0 56 48 0 0 56 48 0 0 56 48 0 0 56 48 0 0 56 48 0 0 56 48 0 0 56 48 0 0 56 48 0 0 56 48 0 0 56 48 0 0 63 240 0 0 63 224 0 0 63 224 0 0])!

aWhiteCursor
	"UIMaskEditor new openOnClass: self andSelector: #aWhiteCursor"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[0 0 0 0 0 0 0 0 32 4 0 0 32 4 0 0 48 4 0 0 16 8 0 0 16 8 0 0 17 136 0 0 25 152 0 0 9 144 0 0 10 80 0 0 10 80 0 0 14 112 0 0 4 32 0 0 0 0 0 0 0 0 0 0])!

aWhiteCursorMask
	"UIMaskEditor new openOnClass: self andSelector: #aWhiteCursorMask"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 1 bitsPerPixel: 1 palette: CoveragePalette monoMaskPalette usingBits: #[0 0 0 0 0 0 0 0 112 14 0 0 112 14 0 0 120 14 0 0 56 28 0 0 59 220 0 0 59 220 0 0 63 252 0 0 31 248 0 0 31 248 0 0 31 248 0 0 31 248 0 0 14 112 0 0 4 32 0 0 0 0 0 0])! !

ApplicationModel subclass: #Chess
	instanceVariableNames: 'chessBoard chessView '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'App 3'!


!Chess methodsFor: 'actions'!

endGame
	| choice |
	choice := Dialog
				choose: ''
				labels: (Array
						with: 'Quit'
						with: 'Start new game'
						with: 'Return to game')
				values: #(#quit #newGame #ignore)
				default: #ignore.
	choice == #ignore ifFalse: [self perform: choice]!

help
	(SimpleHelp helpString: 'To make a move, select your piece by clicking it with the left button.\Then select destination square by clicking it with the left button again.\\Note that the text at the bottom indicates which player moves next.\\When a check occurs, the word CHECK appears at the bottom in red. It disappears when the opponent makes an appropriate move.\\Some moves, such as capturing by pawn, castling, and capturing en passant, are not available. Their implementation is left as an exercise:)' withCRs) open!

newGame
	chessBoard newGame.
	chessView invalidate.
	(self builder componentAt: #black) beInvisible.
	(self builder componentAt: #white) beVisible!

quit
	self closeRequest! !

!Chess methodsFor: 'initialization'!

initialize
	chessBoard := ChessBoard newFrom: self.
	chessView := ChessView new initialize.
	chessView model: chessBoard! !

!Chess methodsFor: 'accessing'!

chessView
	^chessView!

player: aPlayer 
	| otherPlayer |
	otherPlayer := aPlayer == #white
				ifTrue: [#black]
				ifFalse: [#white].
	(self builder componentAt: otherPlayer) beInvisible.
	(self builder componentAt: aPlayer) beVisible! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Chess class
	instanceVariableNames: ''!


!Chess class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Chess' 
			#min: #(#Point 288 353 ) 
			#max: #(#Point 288 353 ) 
			#bounds: #(#Rectangle 256 124 544 477 ) ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#ArbitraryComponentSpec 
					#layout: #(#Rectangle 16 16 272 272 ) 
					#name: #chessView 
					#component: #chessView ) 
				#(#LabelSpec 
					#layout: #(#Point 7 284 ) 
					#label: 'Current player' ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 184 320 272 344 ) 
					#model: #endGame 
					#label: 'End game' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 184 288 272 312 ) 
					#model: #help 
					#label: 'Help' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#Point 6 306 ) 
					#name: #white 
					#label: 'white' 
					#style: #large ) 
				#(#LabelSpec 
					#layout: #(#Point 6 306 ) 
					#name: #black 
					#flags: 16 
					#label: 'black' 
					#style: #large ) 
				#(#LabelSpec 
					#layout: #(#Point 104 312 ) 
					#name: #check 
					#flags: 16 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #red ) ) 
					#label: 'CHECK' 
					#style: #large ) ) ) )! !

