Stream subclass: #CircularBuffer
	instanceVariableNames: 'array firstElement firstLocation isEmpty capacity '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 10'!
CircularBuffer comment:
'I implement a circular buffer using a fixed-size array.

Instance variables

firstLocation	<Integer>	location of the next available location
firstElement	<Integer>	location of the next element to be retrieved
array		<Array>		stores buffer elements
isEmpty	<Boolean>	is the buffer empty?
capacity	<Integer>	capacity of Array'!


!CircularBuffer methodsFor: 'initialization'!

initialize: anInteger 
	array := Array new: anInteger.
	capacity := array size.
	firstElement := 1.
	firstLocation := 1.
	isEmpty := true! !

!CircularBuffer methodsFor: 'testing'!

isEmpty
	^isEmpty!

isFull
	"Are all slots occupied?"

	^(firstElement = firstLocation) and: [isEmpty not]! !

!CircularBuffer methodsFor: 'private'!

moveFirstElementIndex
	"Element was removed, update firstElement."

	firstElement := (firstElement rem: capacity) + 1.
	isEmpty := firstElement = firstLocation!

moveFirstLocationIndex
	"An element has been added, 'increment' firstIndex."

	firstLocation := (firstLocation rem: capacity) + 1.
	isEmpty := false! !

!CircularBuffer methodsFor: 'accessing'!

next
	"Return next element and move pointer, return nil if empty."

	^isEmpty
		ifTrue: [nil]
		ifFalse: 
			[| el |
			el := array at: firstElement.
			self moveFirstElementIndex.
			el]!

nextPut: anObject 
	"Add new element if there is room, otherwise execute exception block."

	self isFull ifTrue: [^self error: 'Buffer is full'].
	array at: firstLocation put: anObject.
	self moveFirstLocationIndex.
	^anObject!

size
	"Answer number of elements current stored in buffer."

	^firstIndex >= lastIndex
		ifTrue: [firstIndex - lastIndex]
		ifFalse: [array size - lastIndex + firstIndex]! !

!CircularBuffer methodsFor: 'enumerating'!

do: aBlock 
	"Evaluate aBlock with each of the receiver's elements as the argument."

	self isEmpty ifTrue: [^self].
	firstElement >= firstLocation
		ifTrue: 
			[firstElement to: capacity do: [:index | aBlock value: (array at: index)].
			1 to: firstLocation - 1 do: [:index | aBlock value: (array at: index)]]
		ifFalse: [firstElement to: firstLocation - 1 do: [:index | aBlock value: (array at: index)]]! !

!CircularBuffer methodsFor: 'printing'!

printOn: aStream 
	"Append to the argument aStream a sequence of characters 
	that identifies the collection."

	| first |
	aStream print: self class; nextPutAll: ' ('.
	first := true.
	self
		do: 
			[:element | 
			first
				ifTrue: [first := false]
				ifFalse: [aStream space].
			element printOn: aStream].
	aStream nextPut: $)! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

CircularBuffer class
	instanceVariableNames: ''!


!CircularBuffer class methodsFor: 'creation'!

new: anInteger 
	^self basicNew initialize: anInteger! !

