ApplicationModel subclass: #AlarmToolTest
	instanceVariableNames: 'alarms '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 13'!
AlarmToolTest comment:
'I implement the basis of a preliminary test of AlarmTool ideas.

Instance variables

alarms	<OrderedCollection>		elements are #(time comment process> arrays'!


!AlarmToolTest methodsFor: 'initialization'!

initialize
	alarms := OrderedCollection new! !

!AlarmToolTest methodsFor: 'adding'!

addAlarm: anArray 
	"Create and schedule a process corresponding to alarm (represented by anArray with time and 
	description), and the complete alarm information to alarm collection."

	| process |
	process := self scheduleAlarmProcessWith: anArray.	"Concatenate 2-element array into 3-element array."
	alarms add: (anArray copyWith: process)! !

!AlarmToolTest methodsFor: 'scheduling'!

scheduleAlarmProcessWith: anArray 
	"Create and schedule a process with parameters described by anArray."

	| block |
	block := 
			[:seconds :comment | 
			(Delay forMilliseconds: seconds * 1000) wait.
			alarms removeFirst.	"Time expired, remove yourself from the list."
			Dialog warn: 'Alarm: ' , comment].
	^(block newProcessWithArguments: anArray)
		priority: Processor userInterruptPriority; resume! !

!AlarmToolTest methodsFor: 'removing'!

removeAlarm: anArray 
	"Remove alarm from collection and terminate its alarm process."

	| anAlarm |
	anAlarm := alarms detect: [:el | (el at: 1)
					= (anArray at: 1)].
	(anAlarm at: 3) terminate.
	alarms remove: anAlarm! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

AlarmToolTest class
	instanceVariableNames: ''!


!AlarmToolTest class methodsFor: 'creation'!

new
	^super new initialize! !

ApplicationModel subclass: #TrainSimulation
	instanceVariableNames: 'trainSemaphoreColor truckSemaphoreColor delay OKToClose enteringCrossing trainCrossingLimit truckCrossingLimit train truck layout semaphore trainTrackLength truckTrackLength trainTrackStart truckTrackStart trainProcess truckProcess speed '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 13'!


!TrainSimulation methodsFor: 'actions'!

changeRequest
	OKToClose
		ifTrue: [^true]
		ifFalse: 
			[Dialog warn: 'Please click Stop to stop running processes before closing.'.
			^false]!

go
	OKToClose := false.
	trainProcess := [
			[train move.
			Processor yield] repeat] fork.
	truckProcess := [
			[truck move.
			Processor yield] repeat] fork!

stop
	truckProcess terminate.
	trainProcess terminate.
	OKToClose := true! !

!TrainSimulation methodsFor: 'accessing'!

delay
	^speed value!

layout
	^layout!

semaphore
	^semaphore!

train
	^train!

trainCrossingLimit

	^trainCrossingLimit!

trainDirection
	^train direction!

trainLength
	^train vehicleLength!

trainPosition
	^train position!

trainSemaphoreColor

	^trainSemaphoreColor!

trainTrackLength

	^trainTrackLength!

trainTrackLength: aValue

	trainTrackLength := aValue!

trainTrackStart

	^trainTrackStart!

trainTrackStart: aValue

	trainTrackStart := aValue!

truck
	^truck!

truckCrossingLimit

	^truckCrossingLimit!

truckDirection
	^truck direction!

truckLength
	^truck vehicleLength!

truckPosition
	^truck position!

truckSemaphoreColor

	^truckSemaphoreColor!

truckTrackLength

	^truckTrackLength!

truckTrackLength: aValue

	truckTrackLength := aValue!

truckTrackStart

	^truckTrackStart!

truckTrackStart: aValue

	truckTrackStart := aValue! !

!TrainSimulation methodsFor: 'displaying'!

display: aSymbol 
	layout display: aSymbol!

semaphores: aColorValue for: aSymbol 
	aSymbol == #train
		ifTrue: 
			[layout displayTruckSemaphoresWithColor: aColorValue.
			truckSemaphoreColor := aColorValue]
		ifFalse: 
			[layout displayTrainSemaphoresWithColor: aColorValue.
			trainSemaphoreColor := aColorValue]! !

!TrainSimulation methodsFor: 'initialization'!

initialize
	trainTrackLength := 160.
	truckTrackLength := 130.
	trainTrackStart := 15 @ 80.
	truckTrackStart := 90 @ 25.
	trainCrossingLimit := 25.
	truckCrossingLimit := 20.
	truck := Truck newOnModel: self.
	train := Train newOnModel: self.
	semaphore := Semaphore new.
	semaphore signal.
	trainSemaphoreColor := ColorValue green.
	truckSemaphoreColor := ColorValue green.
	layout := LayoutView new.
	layout model: self.
	speed := 25 asValue.
	OKToClose := true! !

!TrainSimulation methodsFor: 'aspects'!

speed
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^speed isNil
		ifTrue:
			[speed := 5 asValue]
		ifFalse:
			[speed]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TrainSimulation class
	instanceVariableNames: ''!


!TrainSimulation class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Train simulation - Semaphores' 
			#min: #(#Point 277 283 ) 
			#max: #(#Point 277 283 ) 
			#bounds: #(#Rectangle 262 159 539 442 ) 
			#isEventDriven: true ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#ArbitraryComponentSpec 
					#layout: #(#Rectangle 5 6 272 204 ) 
					#component: #layout ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 215 214 267 239 ) 
					#model: #go 
					#label: 'Go' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 215 247 267 272 ) 
					#model: #stop 
					#label: 'Stop' 
					#defaultable: true ) 
				#(#SliderSpec 
					#layout: #(#Rectangle 8 247 171 272 ) 
					#model: #speed 
					#orientation: #horizontal 
					#start: 150 
					#stop: 1 ) 
				#(#LabelSpec 
					#layout: #(#Point 69 215 ) 
					#label: 'Speed' ) 
				#(#LabelSpec 
					#layout: #(#Point 8 226 ) 
					#label: 'Slow' ) 
				#(#LabelSpec 
					#layout: #(#Point 143 226 ) 
					#label: 'Fast' ) ) ) )! !

Object subclass: #Alarm
	instanceVariableNames: 'time description process '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 13'!


!Alarm methodsFor: 'printing'!

displayString
	^time printString!

printIt
	^time printString! !

!Alarm methodsFor: 'accessing'!

description

	^description!

description: aValue

	description := aValue!

process

	^process!

process: aValue

	process := aValue!

processs

	^processs!

processs: aValue

	processs := aValue!

time

	^time!

time: aValue

	time := aValue! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Alarm class
	instanceVariableNames: ''!


!Alarm class methodsFor: 'creation'!

time: aTime desc: aString process: aProcess 
	^(self new) time: aTime; description: aString; process: aProcess! !

Controller subclass: #LayoutController
	instanceVariableNames: 'endArea isDragging crossArrowsCursor horizontalArrowsCursor verticalArrowsCursor tracking newCursorPosition oldCursorPosition '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 13'!


!LayoutController methodsFor: 'events'!

mouseMovedEvent: event 
	| areaCursor |
	isDragging
		ifTrue: [oldCursorPosition = (newCursorPosition := self sensor cursorPointFor: event)
				ifFalse: 
					[self perform: tracking.
					oldCursorPosition := newCursorPosition]]
		ifFalse: 
			[areaCursor := self hotSpotAreaCursorFor: (self sensor cursorPointFor: event).
			Cursor currentCursor: areaCursor]!

redButtonPressedEvent: event 
	tracking isNil
		ifFalse: 
			[isDragging := true.
			oldCursorPosition := self sensor cursorPointFor: event]!

redButtonReleasedEvent: event 
	isDragging := false! !

!LayoutController methodsFor: 'accessing - dragging'!

horizontalTrackInvalidationAreaForTrack: anArray
	^Rectangle 
			origin: (anArray at: 1) + (-30 @ -30)
			corner: (anArray at: 2) + (30 @ 30)! !

!LayoutController methodsFor: 'accessing'!

bottomVerticalTrackAreaEndpoint
	^endArea translatedBy: (self truckTrack at: 2)!

horizontalTrackArea
	| track |
	^Rectangle origin: ((track := self trainTrack) at: 1)
			+ (2 @ -2) corner: (track at: 2)
			- (2 @ -2)!

leftHorizontalTrackAreaEndpoint
	^endArea translatedBy: (self trainTrack at: 1)!

rightHorizontalTrackAreaEndpoint
	^endArea translatedBy: (self trainTrack at: 2)!

topVerticalTrackAreaEndpoint
	^endArea translatedBy: (self truckTrack at: 1)!

trainTrack
	| start |
	^Array 	with: (start := model trainTrackStart) 
			with: start + (model trainTrackLength @ 0)!

truckTrack
	| start |
	^Array 	with: (start := model truckTrackStart) 
			with: start + (0 @ model truckTrackLength)!

verticalTrackArea
	| track |
	^Rectangle origin: ((track := self truckTrack) at: 1)
			+ (-2 @ 2) corner: (track at: 2)
			+ (2 @ -2)! !

!LayoutController methodsFor: 'dragging'!

bottomVerticalEnd
	"Can drag up or down within subview limits."!

horizontalTrack
	"Can drag in any direction within subview limits. Does not change track length."

	| shift track trackStart trackLength |
	shift := newCursorPosition - oldCursorPosition.
	trackStart := model trainTrackStart + shift.
	model trainTrackStart: trackStart.
	trackLength := model trainTrackLength.
	track := Array with: trackStart with: trackStart + (trackLength @ 0).
	view invalidateRectangle: (self horizontalTrackInvalidationAreaForTrack: track)
		repairNow: true!

leftHorizontalEnd
	"Can drag left or right within subview limits and right of left end. Changes length of track."

	| shift trackStart trackLength track |
	shift := newCursorPosition x - oldCursorPosition x.
	trackStart := model trainTrackStart + (shift @ 0).
	trackLength := model trainTrackLength - shift.
	model trainTrackStart: trackStart.
	model trainTrackLength: trackLength.
	track := Array with: trackStart with: trackStart + (trackLength @ 0).
	view invalidateRectangle: (self horizontalTrackInvalidationAreaForTrack: track)
		repairNow: true!

rightHorizontalEnd
	"Can drag left or right within subview limits and right of left end. Changes length of track."

	| shift trackStart trackLength track |
	shift := newCursorPosition x - oldCursorPosition x.
	trackStart := model trainTrackStart.
	trackLength := model trainTrackLength + shift.
	model trainTrackLength: trackLength.
	track := Array with: trackStart with: trackStart + (trackLength @ 0).
	view invalidateRectangle: (self horizontalTrackInvalidationAreaForTrack: track)
		repairNow: true!

topVerticalEnd
	"Can drag up or down within subview limits."!

verticalTrack
	"Can drag in any direction within subview limits."! !

!LayoutController methodsFor: 'private'!

getTrackingParameterFor: aPoint 
	"Return symbol representing tracking parameter."

	(self bottomVerticalTrackAreaEndpoint containsPoint: aPoint)
		ifTrue: [^tracking := #bottomVerticalEnd].
	(self topVerticalTrackAreaEndpoint containsPoint: aPoint)
		ifTrue: [^tracking := #topVerticalEnd].
	(self verticalTrackArea containsPoint: aPoint)
		ifTrue: [^tracking := #verticalTrack].
	(self leftHorizontalTrackAreaEndpoint containsPoint: aPoint)
		ifTrue: [^tracking := #leftHorizontalEnd].
	(self rightHorizontalTrackAreaEndpoint containsPoint: aPoint)
		ifTrue: [^tracking := #rightHorizontalEnd].
	(self horizontalTrackArea containsPoint: aPoint)
		ifTrue: [^tracking := #horizontalTrack].
	^tracking := nil!

hotSpotAreaCursorFor: aPoint 
	self getTrackingParameterFor: aPoint.
	^self selectCursor!

selectCursor
	(#(#leftHorizontalEnd #rightHorizontalEnd) includes: tracking)
		ifTrue: [^horizontalArrowsCursor].
	(#(#bottomVerticalEnd #topVerticalEnd) includes: tracking)
		ifTrue: [^verticalArrowsCursor].
	(#(#horizontalTrack #verticalTrack) includes: tracking)
		ifTrue: [^crossArrowsCursor].
	^Cursor normal! !

!LayoutController methodsFor: 'initialization'!

initialize
	isDragging := false.
	horizontalArrowsCursor := Cursor
				image: self class horizontalArrows asImage
				mask: self class horizontalArrowsMask asImage
				hotSpot: 8 @ 8
				name: 'horizontal arrows'.
	verticalArrowsCursor := Cursor
				image: self class verticalArrows asImage
				mask: self class verticalArrowsMask asImage
				hotSpot: 8 @ 8
				name: 'horizontal arrows'.
	crossArrowsCursor := Cursor
				image: self class crossArrows asImage
				mask: self class crossArrowsMask asImage
				hotSpot: 8 @ 8
				name: 'horizontal arrows'.
	endArea := -4 @ -3 corner: 4 @ 3! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LayoutController class
	instanceVariableNames: ''!


!LayoutController class methodsFor: 'resources'!

crossArrows
	"UIMaskEditor new openOnClass: self andSelector: #crossArrows"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[1 0 0 0 2 128 0 0 6 192 0 0 2 128 0 0 2 128 0 0 34 136 0 0 126 252 0 0 128 2 0 0 126 252 0 0 34 136 0 0 2 128 0 0 2 128 0 0 6 192 0 0 2 128 0 0 1 0 0 0 0 0 0 0])!

crossArrowsMask
	"UIMaskEditor new openOnClass: self andSelector: #crossArrowsMask"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 1 bitsPerPixel: 1 palette: CoveragePalette monoMaskPalette usingBits: #[1 0 0 0 2 128 0 0 6 192 0 0 6 192 0 0 6 192 0 0 62 248 0 0 126 252 0 0 128 2 0 0 126 252 0 0 62 248 0 0 6 192 0 0 6 192 0 0 6 192 0 0 2 128 0 0 1 0 0 0 0 0 0 0])!

horizontalArrows
	"UIMaskEditor new openOnClass: self andSelector: #horizontalArrows"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 16 16 0 0 32 8 0 0 127 252 0 0 128 2 0 0 127 252 0 0 32 8 0 0 16 16 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0])!

horizontalArrowsMask
	"UIMaskEditor new openOnClass: self andSelector: #horizontalArrowsMask"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 1 bitsPerPixel: 1 palette: CoveragePalette monoMaskPalette usingBits: #[0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 48 24 0 0 127 252 0 0 255 254 0 0 128 2 0 0 255 254 0 0 127 252 0 0 48 24 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0])!

verticalArrows
	"UIMaskEditor new openOnClass: self andSelector: #verticalArrows"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[1 0 0 0 2 128 0 0 6 192 0 0 10 160 0 0 2 128 0 0 2 128 0 0 2 128 0 0 2 128 0 0 2 128 0 0 2 128 0 0 2 128 0 0 10 160 0 0 6 192 0 0 2 128 0 0 1 0 0 0 0 0 0 0])!

verticalArrowsMask
	"UIMaskEditor new openOnClass: self andSelector: #verticalArrowsMaks"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 1 bitsPerPixel: 1 palette: CoveragePalette monoMaskPalette usingBits: #[3 128 0 0 6 192 0 0 14 224 0 0 14 224 0 0 6 192 0 0 6 192 0 0 6 192 0 0 6 192 0 0 6 192 0 0 6 192 0 0 6 192 0 0 14 224 0 0 14 224 0 0 6 192 0 0 3 128 0 0 0 0 0 0])! !

Object subclass: #Vehicle
	instanceVariableNames: 'limit position direction model vehicleLength '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 13'!
Vehicle comment:
'I gather the behaviors of Train and Truck for the purpose of graphical simulation of a toy train.

Instance variables

crossing	<Integer>	position of rail crossing from left (Train) or from top (Truck), a constant
limit		<Integer>	distance of Semaphore from crossing at both ends, a constant
position		<Integer>	current position of fron of vehicle on track: from left (Train) or from top (Truck)
direction 	<Symbol>	direction of motion: #left/#right for Train, #up/#down for Truck
trackLength	<Integer>	length of track in pixels
semaphore	<Semaphore>	Semaphore at crossing. Visible to both Train and Truck'!


!Vehicle methodsFor: 'accessing'!

direction
	^direction!

model: aTrainSimulation 
	model := aTrainSimulation!

position
	^position!

position: anInteger
	position := anInteger!

semaphore
^model semaphore!

vehicleLength
	^vehicleLength! !

!Vehicle methodsFor: 'initialization'!

initialize
	position := 0.
	direction := #incrementing! !

!Vehicle methodsFor: 'testing'!

comingToSemaphore
	"Return true if I have just reached crossing zone."

	| crossing |
	crossing := self crossing.
	^(direction == #incrementing and: [position = (crossing - limit - vehicleLength)])
		or: [direction == #decrementing and: [position = (crossing + limit)]]!

endOfTrack
	"Return true if this is the end of the track. Don't forget length of vehicle."

	^(direction = #decrementing and: [position = 0])
		or: [direction == #incrementing and: [position = (self trackLength - vehicleLength)]]!

leavingSemaphore
	"Return true if I have just reached crossing boundary."

	| crossing |
	crossing := self crossing.
	^(direction == #incrementing and: [position = (crossing + limit)])
		or: [direction == #decrementing and: [position = (crossing - limit - vehicleLength)]]! !

!Vehicle methodsFor: 'moving'!

move
	"Move one step and check whether destination is on crossing limit. If so, take appropriate action."

	self comingToSemaphore
		ifTrue: 
			[model semaphore wait.
			self semaphores: ColorValue red.
			^self moveOneStep].
	self leavingSemaphore
		ifTrue: 
			[model semaphore signal.
			self semaphores: ColorValue green.
			^self moveOneStep].
	self endOfTrack ifTrue: [self turn].
	self moveOneStep!

moveOneStep
	"Move one step in current direction."

	direction == #decrementing
		ifTrue: [position := position - 1]
		ifFalse: [position := position + 1].
	model display: self symbol.
	(Delay forMilliseconds: model delay) wait!

semaphores: aColorValue
	model semaphores: aColorValue for: self symbol!

turn
	"Reached end of track, turn."

	direction == #incrementing
		ifTrue: [direction := #decrementing]
		ifFalse: [direction := #incrementing]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Vehicle class
	instanceVariableNames: ''!


!Vehicle class methodsFor: 'creation'!

newOnModel: aTrainSimulation 
	^(self new model: aTrainSimulation) initialize! !

Vehicle subclass: #Train
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 13'!


!Train methodsFor: 'initialization'!

initialize
	super initialize.
	limit := model trainCrossingLimit.
	vehicleLength := 20! !

!Train methodsFor: 'moving'!

moveOneStep
	super moveOneStep.
	model display: #train!

semaphores: aColorValue 
	model semaphores: aColorValue for: #train! !

!Train methodsFor: 'accessing'!

crossing
	^model truckTrackStart x - model trainTrackStart x!

symbol
	^#train!

trackLength
	^model trainTrackLength! !

Vehicle subclass: #Truck
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 13'!


!Truck methodsFor: 'initialization'!

initialize
	super initialize.
	limit := model truckCrossingLimit.
	vehicleLength := 10! !

!Truck methodsFor: 'moving'!

moveOneStep
	super moveOneStep.
	model display: #truck!

semaphores: aColorValue 
	model semaphores: aColorValue for: #truck! !

!Truck methodsFor: 'accessing'!

crossing
	^model trainTrackStart y - model truckTrackStart y.!

symbol
	^#truck!

trackLength
	^model truckTrackLength! !

ApplicationModel subclass: #StopWatch
	instanceVariableNames: 'time process watch '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 13'!


!StopWatch methodsFor: 'actions'!

reset
self changed: #reset: with: time value!

start
	process := [
			[(Delay forSeconds: 1) wait.
			time value: time value + 1.
			self changed: #moveHand: with: time value] repeat] fork!

stop
	process terminate! !

!StopWatch methodsFor: 'accessing'!

time
	^time!

watch
	^watch! !

!StopWatch methodsFor: 'initialization'!

initialize
	watch := WatchView new.
	watch model: self.
	time := 0 asValue!

postBuildWith: aBuilder 
	watch setParameters! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

StopWatch class
	instanceVariableNames: ''!


!StopWatch class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Stop watch' 
			#min: #(#Point 197 135 ) 
			#max: #(#Point 197 135 ) 
			#bounds: #(#Rectangle 302 233 499 368 ) 
			#isEventDriven: true ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#InputFieldSpec 
					#layout: #(#Rectangle 149 112 180 128 ) 
					#model: #time 
					#alignment: #center 
					#isReadOnly: true 
					#type: #number 
					#formatString: '0' ) 
				#(#ArbitraryComponentSpec 
					#layout: #(#Rectangle 7 8 128 128 ) 
					#name: #watch 
					#component: #watch ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 136 8 192 32 ) 
					#model: #start 
					#label: 'Start' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 136 40 192 64 ) 
					#model: #stop 
					#label: 'Stop' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 136 72 192 96 ) 
					#model: #reset 
					#label: 'Reset' 
					#defaultable: true ) ) ) )! !

ApplicationModel subclass: #AlarmTool
	instanceVariableNames: 'alarms hours amOrPm minutes description alarmList descriptionInForm time '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 13'!
AlarmTool comment:
'I support the Alarm Tool. I hold a collection of alarm objects (time, comment) and a process that is delayed until the first alarm expires, at which time it will open a window with that alarm''s comment. If there are no more alarms, the process terminates, otherwise, it restarts itself with the next alarm.
Since alarms must run even when the Alarm Tool is closed, this information must be held in a global variable (Alarms) which must self-destruct when it is no longer necessary (if the Tool is closed and there are no more alarms pending). Conversely, if the Tool opens, it uses the global variable Alarms if it exists, and creates it if it does not exist.
Note that Alarms remains active and  of time while setting up an alarm'!


!AlarmTool methodsFor: 'private'!

initializeFormForExistingAlarm
	| selection |
	selection := alarmList selection.
	alarms remove: selection.
	selection process terminate.
	self hours value: selection time hours.
	self minutes value: selection time minutes.
	self description value: selection description!

initializeFormForNewAlarm
	self hours value: Time now hours.
	self minutes value: Time now minutes.
	self descriptionInForm value: ''!

scheduleAlarmProcessWith: anArray 
	"Create and schedule a process with alarm defined by first element of alarms."

	| block |
	block := 
			[:maturationTime :comment | 
			(Delay forMilliseconds: maturationTime asSeconds - Time now asSeconds * 1000) wait.
"			alarms removeFirst."
			Dialog warn: ('Alarm at: ' , Time now printString , '\' , comment) withCRs].
	^(block newProcessWithArguments: anArray)
		priority: Processor userInterruptPriority; resume! !

!AlarmTool methodsFor: 'initialization'!

initialize
	"Set alarms for proper kind of SortedCollection."

	alarms := SortedCollection sortBlock: [:x :y | x time < y time].
	alarmList := SelectionInList new.
	alarmList selectionIndexHolder onChangeSend: #changedAlarmSelection to: self.
	alarmListMenuHolder := self menuWithAdd asValue.
	description := '' asValue! !

!AlarmTool methodsFor: 'actions'!

accept

	"This stub method was generated by UIDefiner"

	^self!

cancel

	"This stub method was generated by UIDefiner"

	^self!

close
	self closeRequest!

hoursDec
	"Checking for minimum total time is omitted."

	| h |
	hours value: ((h := hours value - 1) < 0
			ifTrue: [23]
			ifFalse: [h])!

hoursInc
	| h |
	hours value: ((h := hours value + 1) > 23
			ifTrue: [0]
			ifFalse: [h])!

minutesDec
	"Checking for minimum total time is omitted."

	| m |
	minutes value: ((m := minutes value - 1) < 0
			ifTrue: [59]
			ifFalse: [m])!

minutesInc
	| m |
	minutes value: ((m := minutes value + 1) > 59
			ifTrue: [0]
			ifFalse: [m])! !

!AlarmTool methodsFor: 'aspects'!

alarmList
	^alarmList!

amOrPm
	^amOrPm!

description
	^description!

descriptionInForm
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^descriptionInForm isNil
		ifTrue:
			[descriptionInForm := String new asValue]
		ifFalse:
			[descriptionInForm]!

hours
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^hours isNil
		ifTrue:
			[hours := 0 asValue]
		ifFalse:
			[hours]!

minutes
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^minutes isNil
		ifTrue:
			[minutes := 0 asValue]
		ifFalse:
			[minutes]! !

!AlarmTool methodsFor: 'menu'!

addAlarm
	"Prepare parameters for form and send doAddition to do the rest."

	self initializeFormForNewAlarm.
	self doAdd!

doAdd
	"Shared part of add and edit."

	| process |
	(self openDialogInterface: #form)
		ifFalse: [^self].
	time := Time fromSeconds: hours value * 3600 + (minutes value * 60).
	process := self scheduleAlarmProcessWith: (Array with: time with: descriptionInForm value).
	alarms add: (Alarm
			time: time
			desc: descriptionInForm value
			process: process).
	alarmList list: alarms!

editAlarm
	"Prepare parameters for form and send doAddition to do the rest."

	self initializeFormForExistingAlarm.
	self doAdd!

menuWithAdd
	| mb |
	mb := MenuBuilder new.
	mb add: 'add' -> #addAlarm.
	^mb menu!

menuWithAll
	| mb |
	mb := MenuBuilder new.
	mb add: 'add' -> #addAlarm; add: 'edit' -> #editAlarm; add: 'remove' -> #removeAlarm.
	^mb menu!

removeAlarm
	"Remove currently selected alarm from collection and terminate its process."

	| selection |
	selection := alarmList selection.
	selection process terminate.
	alarmList list: (alarms remove: selection; yourself)! !

!AlarmTool methodsFor: 'changes'!

changedAlarmSelection
	| selection |
	selection := self alarmList selection.
	self alarmListMenu value: (selection isNil
			ifTrue: [self menuWithAdd]
			ifFalse: [self menuWithAll]).
	description value: (selection isNil
			ifTrue: [alarms isEmpty
					ifTrue: ['']
					ifFalse: ['Select alarm to see its description.']]
			ifFalse: [selection description])! !

!AlarmTool methodsFor: 'accessing'!

alarmListMenu
	^alarmListMenuHolder! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

AlarmTool class
	instanceVariableNames: ''!


!AlarmTool class methodsFor: 'interface specs'!

form
	"UIPainter new openOnClass: self andSelector: #form"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Alarm specs' 
			#min: #(#Point 288 147 ) 
			#max: #(#Point 288 147 ) 
			#bounds: #(#Rectangle 324 212 612 359 ) 
			#isEventDriven: true ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#LabelSpec 
					#layout: #(#Point 147 14 ) 
					#label: 'minutes' ) 
				#(#LabelSpec 
					#layout: #(#Point 20 13 ) 
					#label: 'hours' ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 69 112 125 136 ) 
					#model: #accept 
					#label: 'Accept' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 141 112 197 136 ) 
					#model: #cancel 
					#label: 'Cancel' 
					#defaultable: true ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 203 14 243 30 ) 
					#model: #minutes 
					#alignment: #center 
					#isReadOnly: true 
					#type: #number ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 60 13 100 29 ) 
					#model: #hours 
					#alignment: #center 
					#isReadOnly: true 
					#type: #number ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 249 5 268 23 ) 
					#flags: 0 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #black ) ) 
					#model: #minutesInc 
					#label: #inc 
					#hasCharacterOrientedLabel: false 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 249 23 268 41 ) 
					#flags: 0 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #black ) ) 
					#model: #minutesDec 
					#label: #dec 
					#hasCharacterOrientedLabel: false 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 107 5 126 23 ) 
					#flags: 0 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #black ) ) 
					#model: #hoursInc 
					#label: #inc 
					#hasCharacterOrientedLabel: false 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 107 23 126 41 ) 
					#flags: 0 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue #black ) ) 
					#model: #hoursDec 
					#label: #dec 
					#hasCharacterOrientedLabel: false 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#Point 20 54 ) 
					#label: 'description' ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 23 74 245 97 ) 
					#model: #descriptionInForm ) ) ) )!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Alarm Tool' 
			#min: #(#Point 264 301 ) 
			#max: #(#Point 264 301 ) 
			#bounds: #(#Rectangle 345 168 609 469 ) 
			#isEventDriven: true ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#SequenceViewSpec 
					#layout: #(#Rectangle 7 28 246 190 ) 
					#model: #alarmList 
					#menu: #alarmListMenu 
					#useModifierKeys: true 
					#selectionType: #highlight ) 
				#(#LabelSpec 
					#layout: #(#Point 7 8 ) 
					#label: 'Existing alarms' ) 
				#(#LabelSpec 
					#layout: #(#Point 7 207 ) 
					#label: 'Description of selected alarm' ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 104 270 156 295 ) 
					#model: #close 
					#label: 'Close' 
					#defaultable: true ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 7 227 246 252 ) 
					#model: #description 
					#isReadOnly: true ) ) ) )! !

!AlarmTool class methodsFor: 'resources'!

dec
	"UIMaskEditor new openOnClass: self andSelector: #dec"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 255 255 192 1 255 255 192 1 255 255 192 1 255 255 192 1 255 255 192 1 247 247 192 1 243 231 192 1 241 199 192 1 248 143 192 1 252 31 192 1 254 63 192 1 255 127 192 1 255 255 192 1 255 255 192 1 255 255 192 1 255 255 192 1 255 255 192 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0])!

inc
	"UIMaskEditor new openOnClass: self andSelector: #inc"

	<resource: #image>
	^CachedImage on: (Image extent: 32@32 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack usingBits: #[0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 3 255 255 192 3 255 255 192 3 255 255 192 3 255 255 192 3 255 255 192 3 255 127 192 3 254 63 192 3 252 31 192 3 248 143 192 3 241 199 192 3 243 231 192 3 247 247 192 3 255 255 192 3 255 255 192 3 255 255 192 3 255 255 192 3 255 255 192 3 255 255 192 3 255 255 192 3 255 255 192 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0])! !

View subclass: #LayoutView
	instanceVariableNames: 'crossing trainStart trainEnd truckStart truckEnd '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 13'!


!LayoutView methodsFor: 'displaying'!

display: aSymbol 
	| aGraphicsContext |
	aGraphicsContext := self graphicsContext.
	aSymbol == #train
		ifTrue: [self displayTrainOn: aGraphicsContext]
		ifFalse: [self displayTruckOn: aGraphicsContext]!

displayOn: aGraphicsContext 
	aGraphicsContext 
		displayLineFrom: self trainStart to: self trainEnd; 
		displayLineFrom: self truckStart to: self truckEnd;
		paint: ColorValue black;
		displayString: 'train' at: self trainStart x + 4 @ self trainStart y - 6;
		displayString: 'truck' at: self truckStart x + 4 @ self truckStart y + 6.
	self displayTrainSemaphoresOn: aGraphicsContext withColor: model trainSemaphoreColor;
		displayTruckSemaphoresOn: aGraphicsContext withColor: model truckSemaphoreColor;
		displayTruck;
		displayTrain!

displayTrain
	self graphicsContext displayRectangle: (0 @ 0 corner: self trainLength @ 5)
		at: self trainStart + (self trainPosition @ -2)!

displayTrainOn: aGraphicsContext 
	| length direction position oldPosition |
	length := model trainLength.
	direction := model trainDirection.
	position := model trainPosition.
	oldPosition := direction == #incrementing
				ifTrue: [position - 1]
				ifFalse: [position + length +1].
	oldPosition = (crossing x - trainStart x)
	ifFalse: [
		aGraphicsContext paint: ColorValue white; 
			displayLineFrom: self trainStart + (oldPosition @ -2) 
				to: self trainStart + (oldPosition @ 2); 
		paint: ColorValue black; 
			displayRectangle: (0 @ 0 corner: length @ 5)
				at: self trainStart + (position @ -2)].
		aGraphicsContext displayLineFrom: self trainStart + (oldPosition @ 0) 
			to: self trainStart + (position @ 0)!

displayTrainSemaphoresOn: aGraphicsContext withColor: aColorValue 
	|  limit rectangle |
	limit := self trainCrossingLimit.
	rectangle := 0 @ 0 corner: 4 @ 4.
	aGraphicsContext 
		paint: aColorValue; 
		displayRectangle: rectangle at: self crossing + (limit negated @ 3);
		displayRectangle: rectangle at: self crossing + (limit - 4 @ -6)!

displayTrainSemaphoresWithColor: aColorValue 
	|  limit rectangle |
	limit := self trainCrossingLimit.
	rectangle := 0 @ 0 corner: 4 @ 4.
	self graphicsContext 
		paint: aColorValue; 
		displayRectangle: rectangle at: self crossing + (limit negated @ 3);
		displayRectangle: rectangle at: self crossing + (limit - 4 @ -6)!

displayTruck
	self graphicsContext displayRectangle: (0 @ 0 corner: 5 @ self truckLength)
		at: self truckStart + (-2 @ self truckPosition )!

displayTruckOn: aGraphicsContext 
	| length direction position oldPosition |
	length := model truckLength.
	direction := model truckDirection.
	position := model truckPosition.
	oldPosition := direction == #incrementing
				ifTrue: [position - 1]
				ifFalse: [position + length + 1].
	oldPosition = (crossing y - truckStart y) 
		ifFalse: [aGraphicsContext paint: ColorValue white;
			displayLineFrom: self truckStart + (-2 @ oldPosition) 
				to: self truckStart + (2 @ oldPosition); 
			paint: ColorValue black; 
			displayRectangle: (0 @ 0 corner: 5 @ length)
				at: self truckStart + (-2 @ position)].
	aGraphicsContext 
		displayLineFrom: self truckStart + (0 @ oldPosition) to: self truckStart + (0 @ position)!

displayTruckSemaphoresOn: aGraphicsContext withColor: aColorValue 
	|  limit rectangle |
	limit := self truckCrossingLimit.
	rectangle := 0 @ 0 corner: 4 @ 4.
	aGraphicsContext 
		paint: aColorValue;
		displayRectangle: rectangle at: self crossing + (-6 @ limit negated);
 		displayRectangle: rectangle at: self crossing + (3 @ (limit - 4))!

displayTruckSemaphoresWithColor: aColorValue 
	|  limit rectangle |
	limit := self truckCrossingLimit.
	rectangle := 0 @ 0 corner: 4 @ 4.
	self graphicsContext 
		paint: aColorValue;
		displayRectangle: rectangle at: self crossing + (-6 @ limit negated);
 		displayRectangle: rectangle at: self crossing + (3 @ (limit - 4))! !

!LayoutView methodsFor: 'controller accessing'!

defaultControllerClass
	^LayoutController! !

!LayoutView methodsFor: 'accessing'!

crossing
	^crossing := self truckStart x @ self trainStart y!

trainCrossingLimit
	^model trainCrossingLimit!

trainEnd
	^trainEnd := model trainTrackStart + (model trainTrackLength @ 0)!

trainLength
	^model trainLength!

trainPosition
	^model trainPosition!

trainStart
	^trainStart := model trainTrackStart!

truckCrossingLimit
	^model truckCrossingLimit!

truckEnd
	^truckEnd := model truckTrackStart + (0 @ model truckTrackLength)!

truckLength
	^model truckLength!

truckPosition
	^model truckPosition!

truckStart
	^truckStart := model truckTrackStart! !

View subclass: #WatchView
	instanceVariableNames: 'center const1 const2 radius1 radius2 '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 13'!


!WatchView methodsFor: 'updating'!

update: aSymbol with: anInteger 
	self perform: aSymbol with: anInteger! !

!WatchView methodsFor: 'initialization'!

setParameters
	| bounds |
	bounds := self bounds.
	center := bounds center.
	radius1 := bounds width - 20 / 2.
	radius2 := radius1 + 6.
	const1 := Float pi / 30.
	const2 := Float pi / 2! !

!WatchView methodsFor: 'displaying'!

displayOn: aGraphicsContext 
	"Display markings and hand."

	| theta |
	theta := 0.
	1 to: 60
		do: 
			[:second | 
			theta := theta + const1.
			aGraphicsContext displayDotOfDiameter: 3 at: center + (Point r: radius2 theta: theta)].
	self drawHand: model time value!

drawHand: anInteger 
	"Draw the hand at position corresponding to anInteger in seconds. Update input field."

	self graphicsContext displayLineFrom: center to: center + (Point r: radius1 theta: anInteger * const1 - const2).
	model time value: anInteger!

eraseHand: anInteger 
	"Erase the hand at position corresponding to anInteger in seconds."

	| backgroundColor |
	backgroundColor := (model builder componentAt: #watch) lookPreferences backgroundColor.
	(self graphicsContext) paint: backgroundColor; displayLineFrom: center to: center + (Point r: radius1 theta: anInteger * const1 - const2)!

moveHand: anInteger 
	"Move the hand to the position corresponding to anInteger in seconds, erase old hand. Reset input 
	field."

	self eraseHand: anInteger - 1.
	self drawHand: anInteger!

reset: anInteger 
	"Reset hand to 0 position."
	self eraseHand: anInteger; drawHand: 0! !

!WatchView methodsFor: 'controller accessing'!

defaultControllerClass
	^NoController! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

WatchView class
	instanceVariableNames: ''!


!WatchView class methodsFor: 'updating'!

update
	self invalidate! !

