ApplicationModel subclass: #ApplicationSelector
	instanceVariableNames: 'currentSelection description '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 6'!


!ApplicationSelector methodsFor: 'actions'!

chess
	"Make Chess the current example and display its short description in the text view."

	currentSelection := Chess.
	description value: 'Current selection: Chess.
This example is a simple chess interface for two players.
Read about it in Appendix 3.'!

driveIn
	"Make DriveInMenu the current example and display its short description in the text view."

	currentSelection := DriveInMenu.
	description value: 'Current selection: Restaurant.
This example is an editable Restaurant menu.
Read about it in Appendix 1.'!

help
	"Open help window with general text on the whole application."

	(SimpleHelp helpString: 'Each of the buttons on the left provides access to an example application.
Clicking one of these buttons, displays information about the selection in the Text View.
If you then click Run, the application will open.') open!

quit
	"Close the window and terminate the application."

	self closeRequest!

run
	"If an example application is selected, open it."

	currentSelection isNil
		ifTrue: [Dialog warn: 'You must select an application first.']
		ifFalse: [currentSelection open]!

tictactoe
	"Make TicTacToe the current example and display its short description in the text view."

	currentSelection := TicTacToe.
	description value: 'Current selection: Tic Tac Toe.
This example is Tic Tac Toe game.
Read about it in Chapter 6.'!

train
"Make TrainSimulation the current example and display its short description in the text view."

	currentSelection := TrainSimulation.
	description value: 'Current selection: Train.
This example is a simulation of a train crossing.
Read about it in Chapter 13.'! !

!ApplicationSelector methodsFor: 'aspects'!

description
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^description isNil
		ifTrue:
			[description := String new asValue]
		ifFalse:
			[description]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ApplicationSelector class
	instanceVariableNames: ''!


!ApplicationSelector class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Application Selector' 
			#min: #(#Point 628 139 ) 
			#max: #(#Point 628 139 ) 
			#bounds: #(#Rectangle 129 258 757 397 ) 
			#isEventDriven: true ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 16 24 112 49 ) 
					#model: #chess 
					#label: 'Chess' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 116 24 212 49 ) 
					#model: #train 
					#label: 'Train' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 16 57 112 82 ) 
					#model: #driveIn 
					#label: 'Restaurant' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 116 57 212 82 ) 
					#model: #tictactoe 
					#label: 'Tic Tac Toe' 
					#defaultable: true ) 
				#(#TextEditorSpec 
					#layout: #(#Rectangle 230 24 617 131 ) 
					#model: #description ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 20 107 72 132 ) 
					#model: #run 
					#label: 'Run' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 89 107 140 132 ) 
					#model: #help 
					#label: 'Help' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#Point 254 3 ) 
					#label: 'Description' ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 157 107 209 132 ) 
					#model: #quit 
					#label: 'Quit' 
					#defaultable: true ) 
				#(#GroupBoxSpec 
					#layout: #(#Rectangle 9 2 217 91 ) 
					#label: 'Applications' ) ) ) )! !

ApplicationModel subclass: #TicTacToe
	instanceVariableNames: 'button1 button2 button3 button4 button5 button6 button7 button8 button9 player '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 6'!


!TicTacToe methodsFor: 'actions'!

button1
	self doButton: (builder componentAt: #button1)!

button2
	self doButton: (builder componentAt: #button2)!

button3
	self doButton: (builder componentAt: #button3)!

button4
	self doButton: (builder componentAt: #button4)!

button5
	self doButton: (builder componentAt: #button5)!

button6
	self doButton: (builder componentAt: #button6)!

button7
	self doButton: (builder componentAt: #button7)!

button8
	self doButton: (builder componentAt: #button8)!

button9
	self doButton: (builder componentAt: #button9)!

reset
	button1 labelString: ''.
	button2 labelString: ''.
	button3 labelString: ''.
	button4 labelString: ''.
	button5 labelString: ''.
	button6 labelString: ''.
	button7 labelString: ''.
	button8 labelString: ''.
	button9 labelString: ''.
	player := 'X'! !

!TicTacToe methodsFor: 'private'!

doButton: aButton
aButton widget label text isEmpty
		ifTrue: 
			[aButton widget labelString: player.
			self endOfGame ifTrue: [^self].
			self newPlayer]
		ifFalse: [Dialog warn: 'This field is already occupied']!

endOfGame
	| end |
	end := (self isPlayer: button1)
				& (self isPlayer: button2) & (self isPlayer: button3) | ((self isPlayer: button4)
					& (self isPlayer: button5) & (self isPlayer: button6)) | ((self isPlayer: button7)
					& (self isPlayer: button8) & (self isPlayer: button9)) | ((self isPlayer: button1)
					& (self isPlayer: button4) & (self isPlayer: button7)) | ((self isPlayer: button2)
					& (self isPlayer: button5) & (self isPlayer: button8)) | ((self isPlayer: button3)
					& (self isPlayer: button6) & (self isPlayer: button9)) | ((self isPlayer: button1)
					& (self isPlayer: button5) & (self isPlayer: button9)) | ((self isPlayer: button3)
					& (self isPlayer: button5) & (self isPlayer: button7)).
	end ifTrue: [Dialog warn: 'Player ' , player , ' wins. Game over.'].
	^end!

isPlayer: button 
	^button label text = player asText!

newPlayer
	player = 'X'
		ifTrue: [player := 'O']
		ifFalse: [player := 'X']! !

!TicTacToe methodsFor: 'initialization'!

initialize
	player := 'X'!

postBuildWith: aBuilder 
	button1 := (aBuilder componentAt: #button1) widget.
	button2 := (aBuilder componentAt: #button2) widget.
	button3 := (aBuilder componentAt: #button3) widget.
	button4 := (aBuilder componentAt: #button4) widget.
	button5 := (aBuilder componentAt: #button5) widget.
	button6 := (aBuilder componentAt: #button6) widget.
	button7 := (aBuilder componentAt: #button7) widget.
	button8 := (aBuilder componentAt: #button8) widget.
	button9 := (aBuilder componentAt: #button9) widget! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TicTacToe class
	instanceVariableNames: ''!


!TicTacToe class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Tic-Tac-Toe' 
			#bounds: #(#Rectangle 253 187 428 387 ) 
			#isEventDriven: true ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 16 32 64 72 ) 
					#name: #button1 
					#flags: 0 
					#model: #button1 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 64 32 112 72 ) 
					#name: #button2 
					#flags: 0 
					#model: #button2 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 112 32 160 72 ) 
					#name: #button3 
					#flags: 0 
					#model: #button3 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 16 72 64 112 ) 
					#name: #button4 
					#flags: 0 
					#model: #button4 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 64 72 112 112 ) 
					#name: #button5 
					#flags: 0 
					#model: #button5 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 112 72 160 112 ) 
					#name: #button6 
					#flags: 0 
					#model: #button6 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 16 112 64 152 ) 
					#name: #button7 
					#flags: 0 
					#model: #button7 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 64 112 112 152 ) 
					#name: #button8 
					#flags: 0 
					#model: #button8 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 112 112 160 152 ) 
					#name: #button9 
					#flags: 0 
					#model: #button9 
					#defaultable: true ) 
				#(#DividerSpec 
					#layout: #(#Rectangle 16 72 160 76 ) ) 
				#(#DividerSpec 
					#layout: #(#Rectangle 16 112 160 116 ) ) 
				#(#DividerSpec 
					#layout: #(#Rectangle 64 32 168 152 ) 
					#orientation: #vertical ) 
				#(#DividerSpec 
					#layout: #(#Rectangle 112 32 216 152 ) 
					#orientation: #vertical ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 32 168 144 192 ) 
					#model: #reset 
					#label: 'Reset' 
					#defaultable: true ) ) ) )! !

