Collection subclass: #NDArray
	instanceVariableNames: 'elements sizes multipliers constant dimensions '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 7'!


!NDArray methodsFor: 'printing'!

printOn: aStream 
"Append to the argument, aStream, the elements of the Array enclosed by parentheses."
| pattern position |
position := 1.
aStream nextPutAll: self class name; nextPutAll: ' ('; cr.
pattern := (Array new: dimensions) atAllPut: 1.
[pattern isNil] whileFalse: 
	 [1 to: pattern size do: [:index | aStream nextPutAll: (pattern at: index) printString , ' '].
	aStream nextPutAll: ':   ', (self atIndices: pattern) printString; cr.
	position := position + 1.
	pattern := self nextPattern: pattern].
aStream nextPut: $)! !

!NDArray methodsFor: 'private'!

indexFor: anArray 
	"Convert series of indices into one linear index."

	| index |
	index := 0.
	multipliers with: anArray do: [:el1 :el2 | index := index + (el1 * el2)].
	^index!

nextPattern: anArray 
"Find the next pattern of indices, return nil if there is none."
| nextPattern |
nextPattern := anArray copy.
"Look for the first element that is smaller than the corresponding dimension."
1 to: dimensions do: 
	 [:index | | value |
		value := anArray at: index.
		value < (sizes at: index)
			ifTrue: [ "Increment the element, reset the preceding, return result."
				nextPattern at: index put: value + 1.
				nextPattern atAll: ( 1 to: index - 1) put: 1.
				^nextPattern]].	
"All patterns printed, we are finished."
^nil! !

!NDArray methodsFor: 'accessing'!

atIndices: anArray 
	"Return element specified by its n indices."

	^elements at: (self indexFor: anArray) - constant!

atIndices: anArray put: anObject
	"Store element specified by its n indices."

	^elements at: (self indexFor: anArray) - constant put: anObject! !

!NDArray methodsFor: 'initialization'!

dimensions: anArayOfSizes 
	"Initialize, including calculation of internal constants from specification."

	| temp |
	dimensions := anArayOfSizes size.
	sizes := anArayOfSizes.
	"The total number of elements is equal to the product of all dimensions."
	elements := Array new: (anArayOfSizes inject: 1 into: [:size :el | el * size]).
	multipliers := Array new: dimensions.
	multipliers at: 1 put: (temp := 1).
	2 to: dimensions
		do: 
			[:index | 
			temp := temp * (anArayOfSizes at: index - 1).
			multipliers at: index put: temp.].
	constant := multipliers inject: -1 into: [:sum :el | el + sum]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NDArray class
	instanceVariableNames: ''!


!NDArray class methodsFor: 'creation'!

dimensions: anArayOfSizes 
	^self new dimensions: anArayOfSizes! !

Collection subclass: #NDArrayRecursive
	instanceVariableNames: 'elements sizes dimensions globalSize pattern '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 7'!


!NDArrayRecursive methodsFor: 'enumerating'!

do: aBlock 
	"Evaluate aBlock with each of the receiver's elements as the argument."

	dimensions = 1
		ifTrue: [elements do: [:element | aBlock value: element]]
		ifFalse: [elements do: [:element | element do: aBlock]]! !

!NDArrayRecursive methodsFor: 'initialization'!

dimensions: anArray globalSize: anInteger 
	"Initialize elements from specification."

	| size |
	sizes := anArray.
	dimensions := anArray size.
	globalSize := anInteger.
	size := anArray at: 1.
	elements := Array new: size.
	dimensions > 1 ifTrue: [1 to: size do: [:index | elements at: index put: (NDArrayRecursive dimensions: (anArray copyFrom: 2 to: dimensions)
					globalSize: anInteger)]]! !

!NDArrayRecursive methodsFor: 'accessing'!

atIndices: anArray 
	^dimensions = 1
		ifTrue: [elements at: (anArray at: 1)]
		ifFalse: [(elements at: (anArray at: 1))
				atIndices: (anArray copyFrom: 2 to: dimensions)]!

atIndices: anArray put: anObject
	^dimensions = 1
		ifTrue: [elements at: (anArray at: 1) put: anObject]
		ifFalse: [(elements at: (anArray at: 1))
				atIndices: (anArray copyFrom: 2 to: dimensions) put: anObject]! !

!NDArrayRecursive methodsFor: 'printing'!

printOn: aStream 
	"Append to the argument, aStream, the elements of the Array enclosed by parentheses."

	dimensions = globalSize
		ifTrue: 
			[aStream nextPutAll: 'NDArrayRecursive'; nextPutAll: ' ('; cr.
			pattern := (Array new: dimensions)
						atAllPut: 1.
			elements do: [:element | element pattern: pattern]].
	dimensions > 1
		ifTrue: [elements
				do: 
					[:element | 
					aStream nextPutAll: element printString.
					self updatePattern]]
		ifFalse: [pattern size
				timesRepeat: 
					[1 to: pattern size do: [:index | aStream nextPutAll: (pattern at: index) printString , ' '].
					aStream nextPutAll: ': ' , (elements at: pattern last) printString; cr.
					self updatePattern]].
	dimensions = globalSize ifTrue: [aStream nextPut: $)]! !

!NDArrayRecursive methodsFor: 'private'!

pattern: anArray 
	pattern := anArray.
	dimensions > 1 ifTrue: [elements do: [:element | element pattern: anArray]]!

updatePattern
	| myIndex |
	myIndex := globalSize - dimensions + 1.
	pattern at: myIndex put: (pattern at: myIndex)
			+ 1.
	pattern atAll: (myIndex + 1 to: globalSize)
		put: 1! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NDArrayRecursive class
	instanceVariableNames: ''!


!NDArrayRecursive class methodsFor: 'creation'!

dimensions: anArray globalSize: anInteger
	"Return new instance with axe dimensions as specified"

	^self new dimensions: anArray globalSize: anInteger!

newWithDimensions: anArray 
	"Return new instance with axe dimensions as specified"

	^self new dimensions: anArray globalSize: anArray size! !

ApplicationModel subclass: #FunctionTable
	instanceVariableNames: 'tableInterface '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 7'!


!FunctionTable methodsFor: 'intialization'!

initialize
	"Create objects underlying the Table widget and calculate its parameters."

	"Create a TwoDList to hold the data and provide the binding to the widget."
	| functions values columnWidths |
	values := TwoDList columns: 5 rows: 10.	"Calculate the data to be displayed, format it, and insert it into the TwoDList."
	functions := #(#cos #sin #ln #log #squared).
	1 to: values columnSize do: [:column | 1 to: values rowSize do: [:row | values atPoint: column @ row put: (PrintConverter print: (row perform: (functions at: column))
					formattedBy: '####.###')]].	"Create the TableInterface on the list."
	tableInterface := TableInterface new selectionInTable: (SelectionInTable with: values).	"Define labels and column widths of the TableInterface."
	columnWidths := 60. "(Array new: values columnSize)
				atAllPut: 60."
	tableInterface columnLabelsArray: #('cos' 'sin' 'ln' 'log' 'square'); rowLabelsArray: #(1 2 3 4 5 6 7 8 9 10); columnWidths: columnWidths! !

!FunctionTable methodsFor: 'aspects'!

tableInterface
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^tableInterface isNil
		ifTrue:
			[tableInterface := TableInterface new selectionInTable: SelectionInTable new]
		ifFalse:
			[tableInterface]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

FunctionTable class
	instanceVariableNames: ''!


!FunctionTable class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Function Table' 
			#bounds: #(#Rectangle 297 84 691 347 ) 
			#isEventDriven: true ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#TableViewSpec 
					#layout: #(#LayoutFrame 0 0.0558376 0 0.0608365 0 0.969543 0 0.980989 ) 
					#model: #tableInterface ) ) ) )! !

TwoDList subclass: #Matrix
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 7'!


!Matrix methodsFor: 'arithmetic'!

scaledBy: aNumber 
	"Return a new matrix whose elements are elements of self multiplied by aNumber."

	"Modify existing collection."
	| newCollection |
	newCollection := collection collect: [:el | el * aNumber].	"Create new instance with the same size and shape and with the new collection."
	^self class
		on: newCollection
		columns: self columnSize
		rows: self rowSize!

sumWith: aMatrix 
	"Calculate a new matrix whose elements are sums of corresponding receiver and argument elements."

	| sum |
	sum := self class columns: self columnSize rows: self rowSize.
	1 to: self rowSize do: [:rowIndex | "Enumerate over rows first."
		1 to: self columnSize
			do: 
				[:columnIndex | 
				| point |
				point := columnIndex @ rowIndex.
				sum at: point put: (self at: point)
						+ (aMatrix at: point)]].
	^sum! !

